/* indexing.h -- Macros having to do with indexing grids
 */

#define BOUNDS 0

/* index checkers */
/* ggg -- Bounds checker */
long ggg(int nx, int ny, int nz, int ix, int iy, int iz);

/* hhh - Bounds checker */
long hhh(int nx, int ny, int ix, int iy);

/* iii -- Bounds checker */
long iii(int nx, int nxs, int nx1, int ny, int nys, int ny1,
	 int nz, int ix, int iy, int iz);

/* jjj -- Bounds checker */
long jjj(int nxs, int nys, int nz, int ix, int iy, int iz);

/* lll -- Bounds checker */
long lll(int nxs, int nys, int nz, int ix, int iy, int iz);

/* kkk - Bounds checker */
long kkk(int nx, int nxs, int nx1, int ny, int nys, int ny1, int ix, int iy);

#if BOUNDS
#define G(ix,iy,iz) ggg(nx,ny,nz,ix,iy,iz)
#define H(ix,iy) hhh(nx,ny,ix,iy)
#define I(ix,iy,iz) iii(nx,nxs,nx1,ny,nys,ny1,nz,ix,iy,iz)
#define J(ix,iy,iz) jjj(nxs,nys,nz,ix,iy,iz)
#define L(ix,iy,iz) lll(nxs,nys,nz,ix,iy,iz)
#define K(ix,iy) kkk(nx,nxs,nx1,ny,nys,ny1,ix,iy)
#else
#define G(ix,iy,iz) ((iz) + nz*(iy) + nz*ny*(ix))
#define H(ix,iy) ((iy) + ny*(ix))
#define I(ix,iy,iz) ((iz) + nz*((iy) + ny1) + nz*ny*((ix) + nx1))
#define J(ix,iy,iz) ((iz) + (nz + 1)*(iy) + (nz + 1)*(nys + 1)*(ix))
#define L(ix,iy,iz) ((iz) + nz*((iy) + 1) + nz*(nys + 2)*((ix) + 1))
#define K(ix,iy) (((iy) + ny1) + ny*((ix) + nx1))
#endif

/* computes a horizontal filter diff on L grid */
#define DFILT(f,ix,iy,iz) (0.25*(f[L(ix + 1,iy,iz)] + f[L(ix - 1,iy,iz)] + f[L(ix,iy + 1,iz)] + f[L(ix,iy - 1,iz)]) - f[L(ix,iy,iz)])

/* computes a 4 point edge average in xy plane on L grid */
#define SQXY(f,ix,iy,iz) ((f[L(ix,iy,iz)] + f[L(ix + 1,iy,iz)] + f[L(ix,iy + 1,iz)] + f[L(ix + 1,iy + 1,iz)])/4.)

/* computes a 4 point edge average in xz plane on L grid */
#define SQXZ(f,ix,iy,iz) ((f[L(ix,iy,iz)] + f[L(ix + 1,iy,iz)] + f[L(ix,iy,iz + 1)] + f[L(ix + 1,iy,iz + 1)])/4.)

/* computes a 4 point edge average in yz plane on L grid */
#define SQYZ(f,ix,iy,iz) ((f[L(ix,iy,iz)] + f[L(ix,iy + 1,iz)] + f[L(ix,iy,iz + 1)] + f[L(ix,iy + 1,iz + 1)])/4.)

/* computes x derivative on L grid, with result on J grid */
#define DX(f,ix,iy,iz) ((SQYZ(f,ix + 1,iy,iz) - SQYZ(f,ix,iy,iz))/dx)

/* computes y derivative on L grid, with result on J grid */
#define DY(f,ix,iy,iz) ((SQXZ(f,ix,iy + 1,iz) - SQXZ(f,ix,iy,iz))/dy)

/* computes z derivative on L grid, with result on J grid */
#define DZ(f,ix,iy,iz) ((SQXY(f,ix,iy,iz + 1) - SQXY(f,ix,iy,iz))/dz)

/* computes an 8 point average corner value from center values on L grid */
#define CU(f,ix,iy,iz) ((f[L(ix,iy,iz)] + f[L(ix + 1,iy,iz)] + f[L(ix,iy + 1,iz)] + f[L(ix,iy,iz + 1)] + f[L(ix + 1,iy + 1,iz)] + f[L(ix + 1,iy,iz + 1)] + f[L(ix,iy + 1,iz + 1)] + f[L(ix + 1,iy + 1, iz + 1)])/8.)

/* computes an 8 point average corner value from center values on J grid */
#define CU2(f,ix,iy,iz) ((f[J(ix,iy,iz)] + f[J(ix + 1,iy,iz)] + f[J(ix,iy + 1,iz)] + f[J(ix,iy,iz + 1)] + f[J(ix + 1,iy + 1,iz)] + f[J(ix + 1,iy,iz + 1)] + f[J(ix,iy + 1,iz + 1)] + f[J(ix + 1,iy + 1, iz + 1)])/8.)

/* computes a 4 point edge average in xy plane on J grid */
#define SQ2XY(f,ix,iy,iz) ((f[J(ix,iy,iz)] + f[J(ix + 1,iy,iz)] + f[J(ix,iy + 1,iz)] + f[J(ix + 1,iy + 1,iz)])/4.)

/* computes a 4 point edge average in xz plane on J grid */
#define SQ2XZ(f,ix,iy,iz) ((f[J(ix,iy,iz)] + f[J(ix + 1,iy,iz)] + f[J(ix,iy,iz + 1)] + f[J(ix + 1,iy,iz + 1)])/4.)

/* computes a 4 point edge average in yz plane on J grid */
#define SQ2YZ(f,ix,iy,iz) ((f[J(ix,iy,iz)] + f[J(ix,iy + 1,iz)] + f[J(ix,iy,iz + 1)] + f[J(ix,iy + 1,iz + 1)])/4.)

/* computes a flux divergence on an L grid */
#define FDIV(fu,fv,fw) ((SQYZ(fu,ix + 1,iy,iz) - SQYZ(fu,ix,iy,iz))/dx + (SQXZ(fv,ix,iy + 1,iz) - SQXZ(fv,ix,iy,iz))/dy + (SQXY(fw,ix,iy,iz + 1) - SQXY(fw,ix,iy,iz))/dz)

/* computes a flux divergence on a J grid */
#define FDIV2(fu,fv,fw) ((SQ2YZ(fu,ix + 1,iy,iz) - SQ2YZ(fu,ix,iy,iz))/dx + (SQ2XZ(fv,ix,iy + 1,iz) - SQ2XZ(fv,ix,iy,iz))/dy + (SQ2XY(fw,ix,iy,iz + 1) - SQ2XY(fw,ix,iy,iz))/dz)
