/* main.c -- Main routine for cumulus ensemble model.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <mpi.h>
#include <fenv.h>
#include <unistd.h>
#include "thermo.h"
#include "indexing.h"
#include "timestep.h"
#include "getprofile.h"
#include "doublebuff.h"
#include "toyrad.h"
#include <quickcandis.h>
#define VERSION "entropy030"
#define PSURF 100000.
#define PIE 3.141592654
#define ALBEDO 0.3
#define SI(iz,j) ((iz) + nj*(j))

/* Define a system switch (gryphon: SYS = 0, UCAR: SYS = 1) */
#define SYS 0

/* Deal with UCAR's incompatible RANDOM, RAND_MAX issue */
#if SYS == 1
#define RAND_MAX 2147483647
#endif

/* declare the feenableexcept function, which is not declared in fenv.h */
#if SYS == 0
int feenableexcept (int excepts);
#endif

/* static variables */
static FILE *startfile;
static FILE *checkfile;
static MPI_Group group_world, mygroup;
static MPI_Comm mycom;
static int *pranks;
static int proc;
static int firstcheck = 0;
static int doprofile, docheckstart;
static int nprocs, nprocs_assigned, mpisize, rank;
static double *meanbuff, *meanbuffout, mask;
static long position, meanbuffsize;
static int gx, gy, nx, ny, nz, nxs, nys, nx1, ny1;
static int igx, igy, ix, iy, iz, izm, it, itstart, jt, kt, nt, mt, mtold, lt;
static long nsize, ssize, i, k, izwtg, rcalcint, radcount, dorad;
static int ranseed;
static int wtg, spectral, entrain, j, nj, bcount;
static double dx, dy, dz, dt, localth, localrh, randomth, randomrh,
  xs, ys, zs, eddy, drh, crain, csnow, cevap, wtermw, wtermi, cld,
  cdrag, rh, tpause, stab, vx1, vx2, vx3, vy1,vy2,
  drelax, ueffmin, mumax, wtgbase, minthgrad,
  radbrk, radcool, filter, lambda, stemp0, dstemp;
static double *thmean, *entmean, *rtmean, *vxmean, *vymean,
  psmean;
static double *thmeans, *entmeans, *rtmeans, *vxmeans, *vymeans, psmeans;
static double x, y, z, time, finetime;
static double thinitpert, rhinitpert, zscale, thperturb,
  rhperturb, rhswitch;
static double *thetagrad;
static double *th0, *ent0, *ents0, *rho0, *rt0, *vx0, *vy0, *sentrad0;
static double *entflux, *entfluxcum, *rtflux, *rtfluxcum, *vxflux,
  *vxfluxcum, *vyflux, *vyfluxcum, *rainflux, *rainfluxcum, *stemp;
static double *entcuflux, *entcufluxcum, *rtcuflux, *rtcufluxcum, *rrcuflux,
  *rrcufluxcum, *vxcuflux, *vxcufluxcum, *vycuflux, *vycufluxcum;
static double *rho, *vx, *vy, *vz, *p, *th, *ent, *ents, *rt, *r, *rc, *rr;
static double *sentrad, *sentradcum, *srtrain, *srtraincum, *thpwtg;
static double *kk, *kv, *sentwtg, *srtwtg, *vzwtg, *dvzwtgdz, *dtemp, *adjmask;
static double *svxwtg, *svywtg, *mu;
static double *sentwtgcum, *srtwtgcum, *thpwtgcum, *svxwtgcum, *svywtgcum;
static double *sine0, *thetaj, *vzwtg2;
static double hscale, brunt, sobel;
static double zbreak;
static char basename[121], profname[121];
static char startfilename[121];
static char checkfilename[121];
static char oldcheckname[125];
static char outcheckname[121];
static char mvstring[300];
  
/* temporary stuff needed for radiation -- no need to put in checkfile */
static double *radflux, *zckm, *pi, *satr;

/* legacy variables related to thetae */
static double *the, *thes, *the0, *thes0;

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* index checkers -- this is used only for debugging */
void bounds_report(char *xxx, int ix, int iy, int iz)
{
  fprintf(stderr,"conv bounds violation: %s; %d %d %d; rank: %d\n",
	  xxx,ix,iy,iz,rank);
  exit(1);
}

/* ggg -- Bounds checker */
long ggg(int nx, int ny, int nz, int ix, int iy, int iz)
{
  if (ix < 0 || iy < 0 || iz < 0 || ix >= nx || iy >= ny || iz >= nz) {
    bounds_report("ggg",ix,iy,iz);
    return 0.;
  } 
  else return iz + nz*iy + nz*ny*ix;
}

/* hhh - Bounds checker */
long hhh(int nx, int ny, int ix, int iy)
{
  if (ix < 0 || iy < 0 || ix >= nx || iy >= ny) {
    bounds_report("hhh",ix,iy,0);
    return 0.;
  } 
  else return iy + ny*ix;
}

/* iii -- Bounds checker */
long iii(int nx, int nxs, int nx1, int ny, int nys, int ny1,
	 int nz, int ix, int iy, int iz)
{
  if (ix < 0 || iy < 0 || iz < 0 || ix >= nxs || iy >= nys || iz >= nz) {
    bounds_report("iii",ix,iy,iz);
    return 0.;
  } 
  else return iz + nz*(iy + ny1) + nz*ny*(ix + nx1);
}

/* jjj -- Bounds checker */
long jjj(int nxs, int nys, int nz, int ix, int iy, int iz)
{
  if (ix < 0 || iy < 0 || iz < 0 || ix > nxs || iy > nys || iz > nz) {
    bounds_report("jjj",ix,iy,iz);
    return 0.;
  } 
  else return iz + (nz + 1)*iy + (nz + 1)*(nys + 1)*ix;
}

/* lll -- Bounds checker */
long lll(int nxs, int nys, int nz, int ix, int iy, int iz)
{
  if (ix < -1 || iy < -1 || iz < 0 || ix > nxs || iy > nys || iz > nz + 1) {
    bounds_report("lll",ix,iy,iz);
    return 0.;
  } 
  else return iz + nz*(iy + 1) + nz*(nys + 2)*(ix + 1);
}

/* kkk - Bounds checker */
long kkk(int nx, int nxs, int nx1, int ny, int nys, int ny1, int ix, int iy)
{
  if (ix < 0 || iy < 0 || ix >= nxs || iy >= nys) {
    bounds_report("kkk",ix,iy,0);
    return 0.;
  } 
  else return iy + ny1 + ny*(ix + nx1);
}

/* initialize_main -- mpi initialization */
static void initialize_main()
{
  
  /* initialize MPI -- nprocs_assigned is the actual number of
   * processes assigned by the machine -- rank is the process
   * number starting with zero */
  MPI_Comm_size(MPI_COMM_WORLD,&nprocs_assigned);
  MPI_Comm_rank(MPI_COMM_WORLD,&rank);
  
  /* make sure we have odd number of actual processes -- Encanto
   * insists on returning an even number of processes
   */
  
  /* we need at least one process! */
  if (nprocs_assigned < 1) {
    if (rank == 0) fprintf(stderr,"conv: can't have less than 1 process\n");
    MPI_Finalize();
    exit(2);
  }
  
  /* ok, we got an even number of processes -- deduct one */
  if ((nprocs_assigned & 1) != 1) {
    nprocs = nprocs_assigned - 1;
  }
  
  /* nope, we got an odd number of processes -- go with it */
  else {
    nprocs = nprocs_assigned;
  }
  
  /* calculate number of slave processes */
  mpisize = nprocs - 1;
  
  /* create the world of needed processes -- omit last of even # procs */
  pranks = (int*)malloc(nprocs*sizeof(int));
  for (proc = 0; proc < nprocs; proc++) pranks[proc] = proc;
  MPI_Comm_group(MPI_COMM_WORLD, &group_world);
  MPI_Group_incl(group_world, nprocs, pranks, &mygroup);
  MPI_Comm_create(MPI_COMM_WORLD, mygroup, &mycom);
  
  /* make unwanted process go away -- this occurs if even # of init procs */
  if (rank == nprocs) {
    MPI_Finalize();
    exit(0);
  }
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* average_main -- do averages over whole domain and send to master process */
static void average_main()
{
  int loop;
  MPI_Status status;

  /* I am a worker or a lone master process */
  if (mpisize == 0 || rank > 0) {
    
    /* compute local worker averages */
    for (iz = 0; iz < nz; iz++) {
      thmeans[iz] = entmeans[iz] = rtmeans[iz] =
	vxmeans[iz] = vymeans[iz] = 0.;
      if (iz == 0) {
	psmeans = 0.;
      }
      
      for (ix = 0; ix < nxs; ix++) {
	for (iy = 0; iy < nys; iy++) {
	  i = I(ix,iy,iz);
	  thmeans[iz] += th[i];
	  entmeans[iz] += ent[i];
	  rtmeans[iz] += rt[i];
	  vxmeans[iz] += vx[i];
	  vymeans[iz] += vy[i];
	  if (iz == 0) {
	    psmeans += p[i];
	  }
	}
      }
      thmeans[iz] = thmeans[iz]/(nxs*nys);
      entmeans[iz] = entmeans[iz]/(nxs*nys);
      rtmeans[iz] = rtmeans[iz]/(nxs*nys);
      vxmeans[iz] = vxmeans[iz]/(nxs*nys);
      vymeans[iz] = vymeans[iz]/(nxs*nys);
      if (iz == 0) {
	psmeans = psmeans/(nxs*nys);
      }
    }
    
    /* send results to master process -- only needed if separate workers */
    if (mpisize > 0) {
      
      /* first pack the local means into a buffer */
      position = 0;
      for (iz = 0; iz < nz; iz++) meanbuff[position++] = thmeans[iz];
      for (iz = 0; iz < nz; iz++) meanbuff[position++] = entmeans[iz];
      for (iz = 0; iz < nz; iz++) meanbuff[position++] = rtmeans[iz];
      for (iz = 0; iz < nz; iz++) meanbuff[position++] = vxmeans[iz];
      for (iz = 0; iz < nz; iz++) meanbuff[position++] = vymeans[iz];
      meanbuff[position] = psmeans;
      
      /* send the buffer to the master process */
      MPI_Send(meanbuff,meanbuffsize,MPI_DOUBLE,0,1,mycom);
    }
  }
  
  /* I am the master process */
  if (rank == 0) {
    if (mpisize == 0) {   /* no separate workers -- simple transfer */
      for (iz = 0; iz < nz; iz++) {
	thmean[iz] = thmeans[iz];
	entmean[iz] = entmeans[iz];
	rtmean[iz] = rtmeans[iz];
	vxmean[iz] = vxmeans[iz];
	vymean[iz] = vymeans[iz];
      }
      psmean = psmeans;
    }
    
    /* loop on workers to bring in buffers and average */
    else {
      
      /* clear the accumulator */
      for (iz = 0; iz < meanbuffsize; iz++) meanbuffout[iz] = 0.;
      
      /* loop over ranks to bring in buffers and add to accumulator */
      for (loop = 1; loop <= mpisize; loop++) {
	MPI_Recv(meanbuff,meanbuffsize,MPI_DOUBLE,loop,1,mycom,&status);
	for (iz = 0; iz < meanbuffsize; iz++) {
	  meanbuffout[iz] += meanbuff[iz];
	}
      }
      
      /* divide by number of workers to get average */
      for (iz = 0; iz < meanbuffsize; iz++) meanbuffout[iz] /= mpisize;
    }
  }
  
  /* receive the global means from the master process */
  if (mpisize > 0) {   /* only needed for separate workers */
    
    /* all processes including root receive broadcast */
    MPI_Bcast(meanbuffout,meanbuffsize,MPI_DOUBLE,0,mycom);
    
    /* unpack global means from buffer */
    position = 0;
    for (iz = 0; iz < nz; iz++) thmean[iz] = meanbuffout[position++];
    for (iz = 0; iz < nz; iz++) entmean[iz] = meanbuffout[position++];
    for (iz = 0; iz < nz; iz++) rtmean[iz] = meanbuffout[position++];
    for (iz = 0; iz < nz; iz++) vxmean[iz] = meanbuffout[position++];
    for (iz = 0; iz < nz; iz++) vymean[iz] = meanbuffout[position++];
    psmean = meanbuffout[position];
  }
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* gather -- put the results of each process together for output */
static void gather(int rank, long size, double *field)
{
  double *buff;
  long ix, iy, iz, i, ib, k, kb, loop, segsize, twodim;
  MPI_Status status;

  /* allocate transfer buffer */
  segsize = size/mpisize;
  buff = doublebuff(segsize);

  /* we need to tell the difference between 2D and 3D fields */
  if (size == ssize) {
    twodim = 1;
  }
  else if (size == nsize) {
    twodim = 0;
  }
  else {
    exit(3);
  }

  /* send the appropriate segment of field to the master process */
  if (rank > 0) {
    if (twodim) {
      kb = 0;
      for (ix = 0; ix < nxs; ix++) {
	for (iy = 0; iy < nys; iy++) {
	  k = K(ix,iy);
	  buff[kb++] = field[k];
	}
      }
    }
    else {
      ib = 0;
      for (ix = 0; ix < nxs; ix++) {
	for (iy = 0; iy < nys; iy++) {
	  for (iz = 0; iz < nz; iz++) {
	    i = I(ix,iy,iz);
	    buff[ib++] = field[i];
	  }
	}
      }
    }
    MPI_Send(buff,segsize,MPI_DOUBLE,0,1,mycom);
  }
      
  /* master process receives the field segment */
  else {

    /* loop over worker processes to pick up data */
    for (loop = 1; loop <= mpisize; loop++) {

      /* we must recompute nx1 and ny1 for the process being received from */
      igy = (loop - 1)/gx + 1;
      igx = loop - gx*(igy - 1);
      if (mpisize > 0) {
	nx1 = nxs*(igx - 1);
	ny1 = nys*(igy - 1);
      }
      else {
	nx1 = ny1 = 0;
      }

    /* receive the stuff from the selected worker */
      MPI_Recv(buff,segsize,MPI_DOUBLE,loop,1,mycom,&status);
      if (twodim) {
	kb = 0;
	for (ix = 0; ix < nxs; ix++) {
	  for (iy = 0; iy < nys; iy++) {
	    k = K(ix,iy);
	    field[k] = buff[kb++];
	  }
	}
      }
      else {
	ib = 0;
	for (ix = 0; ix < nxs; ix++) {
	  for (iy = 0; iy < nys; iy++) {
	    for (iz = 0; iz < nz; iz++) {
	      i = I(ix,iy,iz);
	      field[i] = buff[ib++];
	    }
	  }
	}
      }
    }
  }

  /* free buffer */
  fbuf(buff);
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* gather_main -- gather all stuff if using mpi workers */
static void gather_main()
{
  if (mpisize > 0) {   /* only needed if workers exist */
    gather(rank,nsize,rho);
    gather(rank,nsize,vx);
    gather(rank,nsize,vy);
    gather(rank,nsize,vz);
    gather(rank,nsize,p);
    gather(rank,nsize,th);
    gather(rank,nsize,ent);
    gather(rank,nsize,rt);
    gather(rank,nsize,r);
    gather(rank,nsize,rc);
    gather(rank,nsize,rr);
    gather(rank,nsize,kk);
    gather(rank,nsize,kv);
    gather(rank,nsize,entcuflux);
    gather(rank,nsize,entcufluxcum);
    gather(rank,nsize,rtcuflux);
    gather(rank,nsize,rtcufluxcum);
    gather(rank,nsize,rrcuflux);
    gather(rank,nsize,rrcufluxcum);
    gather(rank,nsize,vxcuflux);
    gather(rank,nsize,vxcufluxcum);
    gather(rank,nsize,vycuflux);
    gather(rank,nsize,vycufluxcum);
    gather(rank,nsize,sentrad);
    gather(rank,nsize,sentradcum);
    gather(rank,nsize,srtrain);
    gather(rank,nsize,srtraincum);
    gather(rank,ssize,entflux);
    gather(rank,ssize,entfluxcum);
    gather(rank,ssize,rtflux);
    gather(rank,ssize,rtfluxcum);
    gather(rank,ssize,vxflux);
    gather(rank,ssize,vxfluxcum);
    gather(rank,ssize,vyflux);
    gather(rank,ssize,vyfluxcum);
    gather(rank,ssize,rainflux);
    gather(rank,ssize,rainfluxcum);
  }
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* allocate_main -- allocate memory for fields used in main */
static void allocate_main()
{
  rho = doublebuff(nsize);
  vx = doublebuff(nsize);
  vy = doublebuff(nsize);
  vz = doublebuff(nsize);
  p = doublebuff(nsize);
  th = doublebuff(nsize);
  ent = doublebuff(nsize);
  ents = doublebuff(nsize);
  rt = doublebuff(nsize);
  r = doublebuff(nsize);
  rc = doublebuff(nsize);
  rr = doublebuff(nsize);
  kk = doublebuff(nsize);
  kv = doublebuff(nsize);
  entcuflux = doublebuff(nsize);
  entcufluxcum = doublebuff(nsize);
  rtcuflux = doublebuff(nsize);
  rtcufluxcum = doublebuff(nsize);
  rrcuflux = doublebuff(nsize);
  rrcufluxcum = doublebuff(nsize);
  vxcuflux = doublebuff(nsize);
  vxcufluxcum = doublebuff(nsize);
  vycuflux = doublebuff(nsize);
  vycufluxcum = doublebuff(nsize);
  sentrad = doublebuff(nsize);
  sentradcum = doublebuff(nsize);
  srtrain = doublebuff(nsize);
  srtraincum = doublebuff(nsize);
  th0 = doublebuff(nz);
  ent0 = doublebuff(nz);
  ents0 = doublebuff(nz);
  rho0 = doublebuff(nz);
  rt0 = doublebuff(nz);
  vx0 = doublebuff(nz);
  vy0 = doublebuff(nz);
  sentrad0 = doublebuff(nz);
  sentwtg = doublebuff(nz);
  srtwtg = doublebuff(nz);
  thpwtg = doublebuff(nz);
  sentwtgcum = doublebuff(nz);
  srtwtgcum = doublebuff(nz);
  thpwtgcum = doublebuff(nz);
  vzwtg = doublebuff(nz);
  dvzwtgdz = doublebuff(nz);
  dtemp = doublebuff(nz);
  adjmask = doublebuff(nz);
  svxwtg = doublebuff(nz);
  svywtg = doublebuff(nz);
  svxwtgcum = doublebuff(nz);
  svywtgcum = doublebuff(nz);
  mu = doublebuff(nz);
  thmean = doublebuff(nz);
  entmean = doublebuff(nz);
  rtmean = doublebuff(nz);
  vxmean = doublebuff(nz);
  vymean = doublebuff(nz);
  thmeans = doublebuff(nz);
  entmeans = doublebuff(nz);
  rtmeans = doublebuff(nz);
  vxmeans = doublebuff(nz);
  vymeans = doublebuff(nz);
  entflux = doublebuff(ssize);
  entfluxcum = doublebuff(ssize);
  rtflux = doublebuff(ssize);
  rtfluxcum = doublebuff(ssize);
  vxflux = doublebuff(ssize);
  vyflux = doublebuff(ssize);
  vxfluxcum = doublebuff(ssize);
  vyfluxcum = doublebuff(ssize);
  rainflux = doublebuff(ssize);
  rainfluxcum = doublebuff(ssize);
  stemp = doublebuff(ssize);
  meanbuff = doublebuff(meanbuffsize);
  meanbuffout = doublebuff(meanbuffsize);
  thetagrad = doublebuff(nz);
  thpwtg = doublebuff(nz);
  vzwtg2 = doublebuff(nz);
  thetaj = doublebuff(nj);
  sine0 = doublebuff(nj*nj);
  
  /* legacy variables having to do with thetae */
  the = doublebuff(nsize);
  thes = doublebuff(nsize);
  the0 = doublebuff(nz);
  thes0 = doublebuff(nz);
  
  /* temporary stuff for radiation calculation -- not needed in checkfile */
  radflux = doublebuff(nz);
  zckm = doublebuff(nz);
  pi = doublebuff(nz);
  satr = doublebuff(nz);
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* hydrostatic -- Compute initial hydrostatic profiles of density,
 * potential temperature, and mixing ratio on cell centers
 * starting with the surface potential temperature and the surface pressure.
 * There are assumptions about the lapse rate and the tropopause height.
 */
void hydrostatic(int nz, double dz, double psurf, double stemp0, double rh,
		 double tpause, double stab, double *th0, double *ent0,
		 double *ents0, double *rho0, double *rt0)
{
  int iz;
  double *p0, *pi0;
  double z, grate, cpi0, cth0;

  /* allocate space */
  p0 = doublebuff(nz + 1);
  pi0 = doublebuff(nz + 1);

  /* integrate the hydrostatic equation to get pressure on cell edges
   * and density and potential temperature on cell centers */
  th0[0] = stemp0;
  pi0[0] = CP*pow(psurf/P_REF,KAPPA);

  /* account for the fact that pi0[0] is half grid below surface */
  pi0[0] += 0.5*(GEE/th0[0])*dz;
  p0[0] = P_REF*pow(pi0[0]/CP,1./KAPPA);
  for (iz = 1; iz <= nz; iz++) {
    pi0[iz] = pi0[iz - 1] - (GEE/th0[iz - 1])*dz;
    p0[iz] = P_REF*pow(pi0[iz]/CP,1./KAPPA);
    if (iz < nz) {
      z = iz*dz;

      /* do second order accurate integration */
      grate = (z < tpause) ? stab*gmoist(th0[iz - 1],pi0[iz]) : 1.2e-2;
      th0[iz] = th0[iz - 1] + grate*dz;
      cth0 = 0.5*(th0[iz] + th0[iz - 1]);
      grate = (z < tpause) ? stab*gmoist(cth0,pi0[iz]) : 1.2e-2;
      th0[iz] = th0[iz - 1] + grate*dz;
    }
  }

  /* get the centered density from the edged pressure using
   * the hydrostatic equation */
  for (iz = 0; iz < nz; iz++) {
    rho0[iz] = -(p0[iz + 1] - p0[iz])/(GEE*dz);

    /* also compute the mixing ratio and entropy */
    cpi0 = 0.5*(pi0[iz + 1] + pi0[iz]);
    rt0[iz] = rh*rs(th0[iz],cpi0);
    ent0[iz] = entcomp(th0[iz], rt0[iz], cpi0);
    ents0[iz] = entscomp(th0[iz], cpi0);
  }

  /* free space */
  fbuf(p0);
  fbuf(pi0);
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* hydrostatic2 -- Compute initial hydrostatic profiles of density and
 * equivalent potential temperature, given potential temperature and
 * mixing ratio.
 */
void hydrostatic2(int nz, double dz, double psurf, double *th0,
		  double *ent0, double *ents0, double *rho0, double *rt0)
{
  int iz;
  double *p0, *pi0;
  double cpi0;

  /* allocate space */
  p0 = doublebuff(nz + 1);
  pi0 = doublebuff(nz + 1);

  /* integrate the hydrostatic equation to get pressure on cell edges
   * and density on cell centers */
  pi0[0] = CP*pow(psurf/P_REF,KAPPA);

  /* account for the fact that pi0[0] is half grid below surface */
  pi0[0] += 0.5*(GEE/th0[0])*dz;
  p0[0] = P_REF*pow(pi0[0]/CP,1./KAPPA);
  for (iz = 1; iz <= nz; iz++) {
    pi0[iz] = pi0[iz - 1] - (GEE/th0[iz - 1])*dz;
    p0[iz] = P_REF*pow(pi0[iz]/CP,1./KAPPA);
  }

  /* get the centered density from the edged pressure using
   * the hydrostatic equation */
  for (iz = 0; iz < nz; iz++) {
    rho0[iz] = -(p0[iz + 1] - p0[iz])/(GEE*dz);

    /* also compute the equivalent potential temperature */
    cpi0 = 0.5*(pi0[iz + 1] + pi0[iz]);
    ent0[iz] = entcomp(th0[iz], rt0[iz], cpi0);
    ents0[iz] = entscomp(th0[iz], cpi0);
  }

  /* free space */
  fbuf(p0);
  fbuf(pi0);
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static void stempcalc(int nx, int ny, double dstemp, double stemp0,
		      double *stemp)
{
  int ix, iy;
  long k;
  double phase;

  for (ix = 0; ix < nx; ix++) {
    for (iy = 0; iy < ny; iy++) {
      k = H(ix,iy);
  
      /* no need to mask the stemp as it is not gathered */
      phase = 6.28318*((double)ix)/((double)(nx + 1));
      stemp[k] = stemp0 - dstemp*cos(phase);
    }
  }
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* getfvar -- Get a double variable from the start file */
double getfvar(char *vname)
{
  char buffer[121], name[121], cvalue[121];

  if (fgets(buffer, 121, startfile) == NULL) {
    fprintf(stderr,"getfvar: unexpected end of file on %s\n",vname);
    exit(4);
  }
  if (sscanf(buffer,"%s %s",name,cvalue) != 2) {
    fprintf(stderr,"getfvar: format error for parameter %s\n",vname);
    exit(5);
  }
  if (strcmp(vname,name) != 0) {
    fprintf(stderr,"getfvar: expecting parameter %s, got %s\n",vname,name);
    exit(6);
  }
  return atof(cvalue);
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* getivar -- Get an integer variable from the start file */
int getivar(char *vname)
{
  char buffer[121], name[121], cvalue[121];

  if (fgets(buffer, 121, startfile) == NULL) {
    fprintf(stderr,"getivar: unexpected end of file on %s\n",vname);
    exit(7);
  }
  if (sscanf(buffer,"%s %s",name,cvalue) != 2) {
    fprintf(stderr,"getivar: format error for parameter %s\n",vname);
    exit(8);
  }
  if (strcmp(vname,name) != 0) {
    fprintf(stderr,"getivar: expecting %s, got %s\n",vname,name);
    exit(9);
  }
  return atoi(cvalue);
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* getsvar -- Get a string variable from the start file */
void getsvar(char *vname,char *cvalue)
{
  char buffer[121], name[121];

  if (fgets(buffer, 121, startfile) == NULL) {
    fprintf(stderr,"getsvar: unexpected end of file on %s\n",vname);
    exit(10);
  }
  if (sscanf(buffer,"%s %s",name,cvalue) != 2) {
    fprintf(stderr,"getsvar: format error for parameter %s\n",vname);
    exit(11);
  }
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* compute mask for multiprocessing -- assumes access to rank,
 * igx, igy, nxs, nys
 */
float makemask(int ix, int iy) {
  float mask;
  
  /* mask zeros initial field segments not used by workers for
     those procs -- this is so final gather can be done by a sum
     reduce */
  if (rank == 0) {
    mask = 1.;
  }
  else {
    
    /* march through x dimension most rapidly -- igx and igy
     * label the x and y processor boxes starting from 1,1
     */
    igy = (rank - 1)/gx + 1;
    igx = rank - gx*(igy - 1);
    if ((ix >= (igx - 1)*nxs) && (ix < igx*nxs) &&
	(iy >= (igy - 1)*nys) && (iy < igy*nys)) {
      mask = 1.;
    }
    else {
      mask = 0.;
    }
  }
  return(mask);
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* write a checkpoint file */
void docheck()
{

  /* if a checkfile has already been written, copy the old one */
  if (firstcheck) {
    sprintf(oldcheckname,"old%s",outcheckname);
    sprintf(mvstring,"mv %s %s\n",outcheckname,oldcheckname);
    system(mvstring);
  }

  /* open the file for write */
  if ((checkfile = fopen(outcheckname, "w")) == NULL) {
    fprintf(stderr,"docheck: can't create checkpoint file %s\n",checkfilename);
    exit(12);
  }

  /* write input parameters */
  fwrite(&rh,sizeof(double),1,checkfile);
  fwrite(&tpause,sizeof(double),1,checkfile);
  fwrite(&stab,sizeof(double),1,checkfile);
  fwrite(&vx1,sizeof(double),1,checkfile);
  fwrite(&vx2,sizeof(double),1,checkfile);
  fwrite(&vx3,sizeof(double),1,checkfile);
  fwrite(&vy1,sizeof(double),1,checkfile);
  fwrite(&vy2,sizeof(double),1,checkfile);
  fwrite(&gx,sizeof(int),1,checkfile);
  fwrite(&gy,sizeof(int),1,checkfile);
  fwrite(&nx,sizeof(int),1,checkfile);
  fwrite(&ny,sizeof(int),1,checkfile);
  fwrite(&nz,sizeof(int),1,checkfile);
  fwrite(&nt,sizeof(int),1,checkfile);
  fwrite(&mt,sizeof(int),1,checkfile);
  fwrite(&lt,sizeof(int),1,checkfile);
  fwrite(&dx,sizeof(double),1,checkfile);
  fwrite(&dy,sizeof(double),1,checkfile);
  fwrite(&dz,sizeof(double),1,checkfile);
  fwrite(&dt,sizeof(double),1,checkfile);
  fwrite(&localth,sizeof(double),1,checkfile);
  fwrite(&localrh,sizeof(double),1,checkfile);
  fwrite(&xs,sizeof(double),1,checkfile);
  fwrite(&ys,sizeof(double),1,checkfile);
  fwrite(&zs,sizeof(double),1,checkfile);
  fwrite(&ranseed,sizeof(int),1,checkfile);
  fwrite(&randomth,sizeof(double),1,checkfile);
  fwrite(&randomrh,sizeof(double),1,checkfile);
  fwrite(&rhswitch,sizeof(double),1,checkfile);
  fwrite(&eddy,sizeof(double),1,checkfile);
  fwrite(&filter,sizeof(double),1,checkfile);
  fwrite(&lambda,sizeof(double),1,checkfile);
  fwrite(&drh,sizeof(double),1,checkfile);
  fwrite(&crain,sizeof(double),1,checkfile);
  fwrite(&csnow,sizeof(double),1,checkfile);
  fwrite(&cevap,sizeof(double),1,checkfile);
  fwrite(&wtermw,sizeof(double),1,checkfile);
  fwrite(&wtermi,sizeof(double),1,checkfile);
  fwrite(&cld,sizeof(double),1,checkfile);
  fwrite(&radcool,sizeof(double),1,checkfile);
  fwrite(&radbrk,sizeof(double),1,checkfile);
  fwrite(&rcalcint,sizeof(int),1,checkfile);
  fwrite(&cdrag,sizeof(double),1,checkfile);
  fwrite(&drelax,sizeof(double),1,checkfile);
  fwrite(&wtg,sizeof(int),1,checkfile);
  fwrite(&entrain,sizeof(int),1,checkfile);
  fwrite(&sobel,sizeof(double),1,checkfile);
  fwrite(&wtgbase,sizeof(double),1,checkfile);
  fwrite(&minthgrad,sizeof(double),1,checkfile);
  fwrite(&spectral,sizeof(int),1,checkfile);
  fwrite(&hscale,sizeof(double),1,checkfile);
  fwrite(&stemp0,sizeof(double),1,checkfile);
  fwrite(&dstemp,sizeof(double),1,checkfile);
  fwrite(&ueffmin,sizeof(double),1,checkfile);
  fwrite(&mumax,sizeof(double),1,checkfile);

  /* write auxiliary parameters */
  fwrite(&nsize,sizeof(long),1,checkfile);
  fwrite(&ssize,sizeof(long),1,checkfile);
  fwrite(&meanbuffsize,sizeof(long),1,checkfile);
  fwrite(&it,sizeof(int),1,checkfile);
  fwrite(&nj,sizeof(int),1,checkfile);
  fwrite(&radcount,sizeof(int),1,checkfile);

  /* write the fields */
  fwrite(rho,sizeof(double),nsize,checkfile);
  fwrite(vx,sizeof(double),nsize,checkfile);
  fwrite(vy,sizeof(double),nsize,checkfile);
  fwrite(vz,sizeof(double),nsize,checkfile);
  fwrite(p,sizeof(double),nsize,checkfile);
  fwrite(th,sizeof(double),nsize,checkfile);
  fwrite(ent,sizeof(double),nsize,checkfile);
  fwrite(ents,sizeof(double),nsize,checkfile);
  fwrite(rt,sizeof(double),nsize,checkfile);
  fwrite(r,sizeof(double),nsize,checkfile);
  fwrite(rc,sizeof(double),nsize,checkfile);
  fwrite(rr,sizeof(double),nsize,checkfile);
  fwrite(kk,sizeof(double),nsize,checkfile);
  fwrite(kv,sizeof(double),nsize,checkfile);
  fwrite(entcuflux,sizeof(double),nsize,checkfile);
  fwrite(entcufluxcum,sizeof(double),nsize,checkfile);
  fwrite(rtcuflux,sizeof(double),nsize,checkfile);
  fwrite(rtcufluxcum,sizeof(double),nsize,checkfile);
  fwrite(rrcuflux,sizeof(double),nsize,checkfile);
  fwrite(rrcufluxcum,sizeof(double),nsize,checkfile);
  fwrite(vxcuflux,sizeof(double),nsize,checkfile);
  fwrite(vxcufluxcum,sizeof(double),nsize,checkfile);
  fwrite(vycuflux,sizeof(double),nsize,checkfile);
  fwrite(vycufluxcum,sizeof(double),nsize,checkfile);
  fwrite(sentrad,sizeof(double),nsize,checkfile);
  fwrite(sentradcum,sizeof(double),nsize,checkfile);
  fwrite(srtrain,sizeof(double),nsize,checkfile);
  fwrite(srtraincum,sizeof(double),nsize,checkfile);
  fwrite(th0,sizeof(double),nz,checkfile);
  fwrite(ent0,sizeof(double),nz,checkfile);
  fwrite(ents0,sizeof(double),nz,checkfile);
  fwrite(rho0,sizeof(double),nz,checkfile);
  fwrite(rt0,sizeof(double),nz,checkfile);
  fwrite(vx0,sizeof(double),nz,checkfile);
  fwrite(vy0,sizeof(double),nz,checkfile);
  fwrite(sentrad0,sizeof(double),nz,checkfile);
  fwrite(sentwtg,sizeof(double),nz,checkfile);
  fwrite(srtwtg,sizeof(double),nz,checkfile);
  fwrite(thpwtg,sizeof(double),nz,checkfile);
  fwrite(sentwtgcum,sizeof(double),nz,checkfile);
  fwrite(srtwtgcum,sizeof(double),nz,checkfile);
  fwrite(thpwtgcum,sizeof(double),nz,checkfile);
  fwrite(vzwtg,sizeof(double),nz,checkfile);
  fwrite(dvzwtgdz,sizeof(double),nz,checkfile);
  fwrite(dtemp,sizeof(double),nz,checkfile);
  fwrite(adjmask,sizeof(double),nz,checkfile);
  fwrite(svxwtg,sizeof(double),nz,checkfile);
  fwrite(svywtg,sizeof(double),nz,checkfile);
  fwrite(svxwtgcum,sizeof(double),nz,checkfile);
  fwrite(svywtgcum,sizeof(double),nz,checkfile);
  fwrite(mu,sizeof(double),nz,checkfile);
  fwrite(thmean,sizeof(double),nz,checkfile);
  fwrite(entmean,sizeof(double),nz,checkfile);
  fwrite(rtmean,sizeof(double),nz,checkfile);
  fwrite(vxmean,sizeof(double),nz,checkfile);
  fwrite(vymean,sizeof(double),nz,checkfile);
  fwrite(thmeans,sizeof(double),nz,checkfile);
  fwrite(entmeans,sizeof(double),nz,checkfile);
  fwrite(rtmeans,sizeof(double),nz,checkfile);
  fwrite(vxmeans,sizeof(double),nz,checkfile);
  fwrite(vymeans,sizeof(double),nz,checkfile);
  fwrite(entflux,sizeof(double),ssize,checkfile);
  fwrite(entfluxcum,sizeof(double),ssize,checkfile);
  fwrite(rtflux,sizeof(double),ssize,checkfile);
  fwrite(rtfluxcum,sizeof(double),ssize,checkfile);
  fwrite(vxflux,sizeof(double),ssize,checkfile);
  fwrite(vyflux,sizeof(double),ssize,checkfile);
  fwrite(vxfluxcum,sizeof(double),ssize,checkfile);
  fwrite(vyfluxcum,sizeof(double),ssize,checkfile);
  fwrite(rainflux,sizeof(double),ssize,checkfile);
  fwrite(rainfluxcum,sizeof(double),ssize,checkfile);
  fwrite(stemp,sizeof(double),ssize,checkfile);
  fwrite(meanbuff,sizeof(double),meanbuffsize,checkfile);
  fwrite(meanbuffout,sizeof(double),meanbuffsize,checkfile);
  fwrite(thetagrad,sizeof(double),nz,checkfile);
  fwrite(vzwtg2,sizeof(double),nz,checkfile);
  fwrite(thetaj,sizeof(double),nj,checkfile);
  fwrite(sine0,sizeof(double),nj*nj,checkfile);
  fwrite(the,sizeof(double),nsize,checkfile);
  fwrite(thes,sizeof(double),nsize,checkfile);
  fwrite(the0,sizeof(double),nz,checkfile);
  fwrite(thes0,sizeof(double),nz,checkfile);

  /* close the checkfile */
  fclose(checkfile);
  firstcheck = 1;
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* check consistency for multiprocessing */
void consistency()
{
  
  /* gx and gy must be positive integers */
  if (gx < 1 || gy < 1) {
    if (rank == 0) fprintf(stderr,"conv: gx and gy must be >= 1\n");
    MPI_Finalize();
    exit(13);
  }
  
  /* for gx or gy > 1, should be even */
  if ((gx > 1 && (2*(gx/2) != gx)) || (gy > 1 && (2*(gy/2) != gy))) {
    if (rank == 0) fprintf(stderr,"conv: if gx,gy > 1, must be even\n");
    MPI_Finalize();
    exit(14);
  }

  /* for a single process we must have gx = gy = 1 */
  if (mpisize == 0 && (gx != 1 || gy != 1)) {
    if (rank == 0) fprintf(stderr,"conv: mpisize is zero but gx or gy != 1\n");
    MPI_Finalize();
    exit(15);
  }
  
  /* gx*gy should equal number of slave processes requested for multiproc */
  if (mpisize > 0 && mpisize != gx*gy) {
    if (rank == 0) fprintf(stderr,"conv: gx*gy (%d) != mpisize (%d)\n",
			   gx*gy,mpisize);
    MPI_Finalize();
    exit(16);
  }
  
  /* for single proc must have gx = gy = 1 */
  if (mpisize == 0 && gx != 1 && gy != 1) {
    if (rank == 0) fprintf(stderr,
			   "conv: must have gx = gy = 1 for single proc\n");
    MPI_Finalize();
    exit(17);
  }
  
  /* gx and gy must divide evenly into nx and ny */
  nxs = nx/gx;
  if (nxs*gx != nx) {
    if (rank == 0) fprintf(stderr,"nx (%d) and gx (%d) inconsistent\n",nx,gx);
    MPI_Finalize();
    exit(18);
  }
  nys = ny/gy;
  if (nys*gy != ny) {
    if (rank == 0) fprintf(stderr,"ny (%d) and gy (%d) inconsistent\n",ny,gy);
    MPI_Finalize();
    exit(19);
  }

  /* everything ok */
  if (rank == 0) fprintf(stderr,"conv: mpisize = %d; gx,gy = %d,%d -- ok\n",
			 mpisize,gx,gy);
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* read a checkpoint file */
void readcheck()
{

  /* open the file for read */
  if ((checkfile = fopen(checkfilename, "r")) == NULL) {
    fprintf(stderr,"readcheck: can't open checkpoint file %s\n",
	    checkfilename);
    exit(20);
  }

  /* read input parameters */
  fread(&rh,sizeof(double),1,checkfile);
  fread(&tpause,sizeof(double),1,checkfile);
  fread(&stab,sizeof(double),1,checkfile);
  fread(&vx1,sizeof(double),1,checkfile);
  fread(&vx2,sizeof(double),1,checkfile);
  fread(&vx3,sizeof(double),1,checkfile);
  fread(&vy1,sizeof(double),1,checkfile);
  fread(&vy2,sizeof(double),1,checkfile);
  fread(&gx,sizeof(int),1,checkfile);
  fread(&gy,sizeof(int),1,checkfile);
  fread(&nx,sizeof(int),1,checkfile);
  fread(&ny,sizeof(int),1,checkfile);
  fread(&nz,sizeof(int),1,checkfile);
  fread(&nt,sizeof(int),1,checkfile);
  fread(&mt,sizeof(int),1,checkfile);
  fread(&lt,sizeof(int),1,checkfile);
  fread(&dx,sizeof(double),1,checkfile);
  fread(&dy,sizeof(double),1,checkfile);
  fread(&dz,sizeof(double),1,checkfile);
  fread(&dt,sizeof(double),1,checkfile);
  fread(&localth,sizeof(double),1,checkfile);
  fread(&localrh,sizeof(double),1,checkfile);
  fread(&xs,sizeof(double),1,checkfile);
  fread(&ys,sizeof(double),1,checkfile);
  fread(&zs,sizeof(double),1,checkfile);
  fread(&ranseed,sizeof(int),1,checkfile);
  fread(&randomth,sizeof(double),1,checkfile);
  fread(&randomrh,sizeof(double),1,checkfile);
  fread(&rhswitch,sizeof(double),1,checkfile);
  fread(&eddy,sizeof(double),1,checkfile);
  fread(&filter,sizeof(double),1,checkfile);
  fread(&lambda,sizeof(double),1,checkfile);
  fread(&drh,sizeof(double),1,checkfile);
  fread(&crain,sizeof(double),1,checkfile);
  fread(&csnow,sizeof(double),1,checkfile);
  fread(&cevap,sizeof(double),1,checkfile);
  fread(&wtermw,sizeof(double),1,checkfile);
  fread(&wtermi,sizeof(double),1,checkfile);
  fread(&cld,sizeof(double),1,checkfile);
  fread(&radcool,sizeof(double),1,checkfile);
  fread(&radbrk,sizeof(double),1,checkfile);
  fread(&rcalcint,sizeof(int),1,checkfile);
  fread(&cdrag,sizeof(double),1,checkfile);
  fread(&drelax,sizeof(double),1,checkfile);
  fread(&wtg,sizeof(int),1,checkfile);
  fread(&entrain,sizeof(int),1,checkfile);
  fread(&sobel,sizeof(double),1,checkfile);
  fread(&wtgbase,sizeof(double),1,checkfile);
  fread(&minthgrad,sizeof(double),1,checkfile);
  fread(&spectral,sizeof(int),1,checkfile);
  fread(&hscale,sizeof(double),1,checkfile);
  fread(&stemp0,sizeof(double),1,checkfile);
  fread(&dstemp,sizeof(double),1,checkfile);
  fread(&ueffmin,sizeof(double),1,checkfile);
  fread(&mumax,sizeof(double),1,checkfile);

  /* read auxiliary parameters */
  fread(&nsize,sizeof(long),1,checkfile);
  fread(&ssize,sizeof(long),1,checkfile);
  fread(&meanbuffsize,sizeof(long),1,checkfile);
  fread(&it,sizeof(int),1,checkfile);
  fread(&nj,sizeof(int),1,checkfile);
  fread(&radcount,sizeof(int),1,checkfile);

  /* allocate space needed for fields */
  allocate_main();

  /* now read the fields */
  fread(rho,sizeof(double),nsize,checkfile);
  fread(vx,sizeof(double),nsize,checkfile);
  fread(vy,sizeof(double),nsize,checkfile);
  fread(vz,sizeof(double),nsize,checkfile);
  fread(p,sizeof(double),nsize,checkfile);
  fread(th,sizeof(double),nsize,checkfile);
  fread(ent,sizeof(double),nsize,checkfile);
  fread(ents,sizeof(double),nsize,checkfile);
  fread(rt,sizeof(double),nsize,checkfile);
  fread(r,sizeof(double),nsize,checkfile);
  fread(rc,sizeof(double),nsize,checkfile);
  fread(rr,sizeof(double),nsize,checkfile);
  fread(kk,sizeof(double),nsize,checkfile);
  fread(kv,sizeof(double),nsize,checkfile);
  fread(entcuflux,sizeof(double),nsize,checkfile);
  fread(entcufluxcum,sizeof(double),nsize,checkfile);
  fread(rtcuflux,sizeof(double),nsize,checkfile);
  fread(rtcufluxcum,sizeof(double),nsize,checkfile);
  fread(rrcuflux,sizeof(double),nsize,checkfile);
  fread(rrcufluxcum,sizeof(double),nsize,checkfile);
  fread(vxcuflux,sizeof(double),nsize,checkfile);
  fread(vxcufluxcum,sizeof(double),nsize,checkfile);
  fread(vycuflux,sizeof(double),nsize,checkfile);
  fread(vycufluxcum,sizeof(double),nsize,checkfile);
  fread(sentrad,sizeof(double),nsize,checkfile);
  fread(sentradcum,sizeof(double),nsize,checkfile);
  fread(srtrain,sizeof(double),nsize,checkfile);
  fread(srtraincum,sizeof(double),nsize,checkfile);
  fread(th0,sizeof(double),nz,checkfile);
  fread(ent0,sizeof(double),nz,checkfile);
  fread(ents0,sizeof(double),nz,checkfile);
  fread(rho0,sizeof(double),nz,checkfile);
  fread(rt0,sizeof(double),nz,checkfile);
  fread(vx0,sizeof(double),nz,checkfile);
  fread(vy0,sizeof(double),nz,checkfile);
  fread(sentrad0,sizeof(double),nz,checkfile);
  fread(sentwtg,sizeof(double),nz,checkfile);
  fread(srtwtg,sizeof(double),nz,checkfile);
  fread(thpwtg,sizeof(double),nz,checkfile);
  fread(sentwtgcum,sizeof(double),nz,checkfile);
  fread(srtwtgcum,sizeof(double),nz,checkfile);
  fread(thpwtgcum,sizeof(double),nz,checkfile);
  fread(vzwtg,sizeof(double),nz,checkfile);
  fread(dvzwtgdz,sizeof(double),nz,checkfile);
  fread(dtemp,sizeof(double),nz,checkfile);
  fread(adjmask,sizeof(double),nz,checkfile);
  fread(svxwtg,sizeof(double),nz,checkfile);
  fread(svywtg,sizeof(double),nz,checkfile);
  fread(svxwtgcum,sizeof(double),nz,checkfile);
  fread(svywtgcum,sizeof(double),nz,checkfile);
  fread(mu,sizeof(double),nz,checkfile);
  fread(thmean,sizeof(double),nz,checkfile);
  fread(entmean,sizeof(double),nz,checkfile);
  fread(rtmean,sizeof(double),nz,checkfile);
  fread(vxmean,sizeof(double),nz,checkfile);
  fread(vymean,sizeof(double),nz,checkfile);
  fread(thmeans,sizeof(double),nz,checkfile);
  fread(entmeans,sizeof(double),nz,checkfile);
  fread(rtmeans,sizeof(double),nz,checkfile);
  fread(vxmeans,sizeof(double),nz,checkfile);
  fread(vymeans,sizeof(double),nz,checkfile);
  fread(entflux,sizeof(double),ssize,checkfile);
  fread(entfluxcum,sizeof(double),ssize,checkfile);
  fread(rtflux,sizeof(double),ssize,checkfile);
  fread(rtfluxcum,sizeof(double),ssize,checkfile);
  fread(vxflux,sizeof(double),ssize,checkfile);
  fread(vyflux,sizeof(double),ssize,checkfile);
  fread(vxfluxcum,sizeof(double),ssize,checkfile);
  fread(vyfluxcum,sizeof(double),ssize,checkfile);
  fread(rainflux,sizeof(double),ssize,checkfile);
  fread(rainfluxcum,sizeof(double),ssize,checkfile);
  fread(stemp,sizeof(double),ssize,checkfile);
  fread(meanbuff,sizeof(double),meanbuffsize,checkfile);
  fread(meanbuffout,sizeof(double),meanbuffsize,checkfile);
  fread(thetagrad,sizeof(double),nz,checkfile);
  fread(vzwtg2,sizeof(double),nz,checkfile);
  fread(thetaj,sizeof(double),nj,checkfile);
  fread(sine0,sizeof(double),nj*nj,checkfile);
  fread(the,sizeof(double),nsize,checkfile);
  fread(thes,sizeof(double),nsize,checkfile);
  fread(the0,sizeof(double),nz,checkfile);
  fread(thes0,sizeof(double),nz,checkfile);

  /* close the checkfile */
  fclose(checkfile);

  /* check consistency and compute nxs and nys */
  consistency();

  /* compute the starting loop index */
  itstart = it + 1;

  /* mask the appropriate fields if a worker process */
  if (rank > 0) {
    for (ix = 0; ix < nx; ix++) {
      for (iy = 0; iy < ny; iy++) {
      
      /* mask zeros initial field segments not used by workers for
	 those procs -- this is so final gather can be done by a sum
	 reduce */
	mask = makemask(ix,iy);
	for (iz = 0; iz < nz; iz++) {
	  i = G(ix,iy,iz);
	  vx[i] *= mask;
	  vy[i] *= mask;
	  vz[i] *= mask;
	  th[i] *= mask;
	  rho[i] *= mask;
	  rt[i] *= mask;
	  p[i] *= mask;
	  ent[i] *= mask;
	  ents[i] *= mask;
	  kk[i] *= mask;
	  kv[i] *= mask;
	  entcuflux[i] *= mask;
	  entcufluxcum[i] *= mask;
	  rtcuflux[i] *= mask;
	  rtcufluxcum[i] *= mask;
	  rrcuflux[i] *= mask;
	  rrcufluxcum[i] *= mask;
	  vxcuflux[i] *= mask;
	  vxcufluxcum[i] *= mask;
	  vycuflux[i] *= mask;
	  vycufluxcum[i] *= mask;
	  sentrad[i] *= mask;
	  sentradcum[i] *= mask;
	  srtrain[i] *= mask;
	  srtraincum[i] *= mask;
	  r[i] *= mask;
	  rc[i] *= mask;
	  rr[i] *= mask;
	}
	k = H(ix,iy);
	entflux[k] *= mask;
	entfluxcum[k] *= mask;
	rtflux[k] *= mask;
	rtfluxcum[k] *= mask;
	vxflux[k] *= mask;
	vxfluxcum[k] *= mask;
	vyflux[k] *= mask;
	vyfluxcum[k] *= mask;
	rainflux[k] *= mask;
	rainfluxcum[k] *= mask;
      }
    }
  }
}


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* docandis -- Make a Candis file */
static void docandis(char *basename, char *profname, char *startfilename,
		     int it, int nx, int ny, int nz, int nj,
		     double dx, double dy, double dz, double dt,
		     double time, double *rho, double *th,
		     double *ent, double *ents, double *rt,
		     double *r, double *rc, double *rr, double *vx,
		     double *vy, double *vz, double *p, double *kk,
		     double *kv, double *entcuflux, double *entcufluxcum,
		     double *rtcuflux, double *rtcufluxcum,
		     double *rrcuflux, double *rrcufluxcum,
		     double *vxcuflux, double *vxcufluxcum,
		     double *vycuflux, double *vycufluxcum,
		     double *sentrad, double *sentradcum,
		     double *srtrain, double *srtraincum,
		     double *th0, double *ent0, double *ents0, double *rho0,
		     double *rt0, double *vx0, double *vy0, double *sentrad0,
		     double *sentwtg, double *srtwtg, double *thpwtg,
		     double *sentwtgcum, double *srtwtgcum, double *thpwtgcum,
		     double *vzwtg, double *dvzwtgdz,
		     double *adjmask, double *svxwtg, double *svxwtgcum,
		     double *svywtg, double *svywtgcum, double *mu,
		     double *entflux, double *entfluxcum,
		     double *rtflux, double *rtfluxcum,
		     double *vxflux, double *vxfluxcum,
		     double *vyflux, double *vyfluxcum,
		     double *rainflux, double *rainfluxcum, double *stemp,
		     double brunt, double *thetaj, double *vzwtg2,
		     double *the, double *thes, double *the0, double *thes0)
{
  int handle, dhx, dhy, dhz, dhj;
  int i,ix,iy,iz;
  char cline[81];

  /* compute thetae-related variables */
  for (iz = 0; iz < nz; iz++) {
    for (ix = 0; ix < nx; ix++) {
      for (iy = 0; iy < ny; iy++) {
	i = G(ix,iy,iz);
	the[i] = thecomp(ent[i]);
	thes[i] = thecomp(ents[i]);
      }
    }
    the0[iz] = thecomp(ent0[iz]);
    thes0[iz] = thecomp(ents0[iz]);
  }

  /* open a new candis file with comments */
  handle = candiscreate(basename, it);
  sprintf(cline,"conv: %s",VERSION);
  candiscomment(handle,cline);
  sprintf(cline,"  startfile = %s",startfilename);
  candiscomment(handle,cline);
  sprintf(cline,"  profname = %s",profname);
  candiscomment(handle,cline);

  /* dimensions */
  dhx = candisdim(handle, "x", nx, 0., dx);
  dhy = candisdim(handle, "y", ny, 0., dy);
  dhz = candisdim(handle, "z", nz, 0., dz);
  dhj = candisdim(handle, "j", nj, 0., 1.);

  /* variable fields */
  dcandisfld(handle, "time", 0, 0, 0, 0, 0, &time);
  dcandisfld(handle, "brunt", 0, 0, 0, 0, 0, &brunt);
  dcandisfld(handle, "th0", 1, dhz, 0, 0, 0, th0);
  dcandisfld(handle, "ent0", 1, dhz, 0, 0, 0, ent0);
  dcandisfld(handle, "ents0", 1, dhz, 0, 0, 0, ents0);
  dcandisfld(handle, "rho0", 1, dhz, 0, 0, 0, rho0);
  dcandisfld(handle, "rt0", 1, dhz, 0, 0, 0, rt0);
  dcandisfld(handle, "vx0", 1, dhz, 0, 0, 0, vx0);
  dcandisfld(handle, "vy0", 1, dhz, 0, 0, 0, vy0);
  dcandisfld(handle, "sentrad0", 1, dhz, 0, 0, 0, sentrad0);
  dcandisfld(handle, "sentwtg", 1, dhz, 0, 0, 0, sentwtg);
  dcandisfld(handle, "sentwtgcum", 1, dhz, 0, 0, 0, sentwtgcum);
  dcandisfld(handle, "srtwtg", 1, dhz, 0, 0, 0, srtwtg);
  dcandisfld(handle, "srtwtgcum", 1, dhz, 0, 0, 0, srtwtgcum);
  dcandisfld(handle, "thpwtg", 1, dhz, 0, 0, 0, thpwtg);
  dcandisfld(handle, "thpwtgcum", 1, dhz, 0, 0, 0, thpwtgcum);
  dcandisfld(handle, "vzwtg", 1, dhz, 0, 0, 0, vzwtg);
  dcandisfld(handle, "dvzwtgdz", 1, dhz, 0, 0, 0, dvzwtgdz);
  dcandisfld(handle, "adjmask", 1, dhz, 0, 0, 0, adjmask);
  dcandisfld(handle, "vzwtg2", 1, dhz, 0, 0, 0, vzwtg2);
  dcandisfld(handle, "svxwtg", 1, dhz, 0, 0, 0, svxwtg);
  dcandisfld(handle, "svxwtgcum", 1, dhz, 0, 0, 0, svxwtgcum);
  dcandisfld(handle, "svywtg", 1, dhz, 0, 0, 0, svywtg);
  dcandisfld(handle, "svywtgcum", 1, dhz, 0, 0, 0, svywtgcum);
  dcandisfld(handle, "mu", 1, dhz, 0, 0, 0, mu);
  dcandisfld(handle, "thetaj", 1, dhj, 0, 0, 0, thetaj);
  dcandisfld(handle, "entflux", 2, dhx, dhy, 0, 0, entflux);
  dcandisfld(handle, "entfluxcum", 2, dhx, dhy, 0, 0, entfluxcum);
  dcandisfld(handle, "rtflux", 2, dhx, dhy, 0, 0, rtflux);
  dcandisfld(handle, "rtfluxcum", 2, dhx, dhy, 0, 0, rtfluxcum);
  dcandisfld(handle, "vxflux", 2, dhx, dhy, 0, 0, vxflux);
  dcandisfld(handle, "vxfluxcum", 2, dhx, dhy, 0, 0, vxfluxcum);
  dcandisfld(handle, "vyflux", 2, dhx, dhy, 0, 0, vyflux);
  dcandisfld(handle, "vyfluxcum", 2, dhx, dhy, 0, 0, vyfluxcum);
  dcandisfld(handle, "rainflux", 2, dhx, dhy, 0, 0, rainflux);
  dcandisfld(handle, "rainfluxcum", 2, dhx, dhy, 0, 0, rainfluxcum);
  dcandisfld(handle, "stemp", 2, dhx, dhy, 0, 0, stemp);
  dcandisfld(handle, "rho", 3, dhx, dhy, dhz, 0, rho);
  dcandisfld(handle, "th", 3, dhx, dhy, dhz, 0, th);
  dcandisfld(handle, "ent", 3, dhx, dhy, dhz, 0, ent);
  dcandisfld(handle, "ents", 3, dhx, dhy, dhz, 0, ents);
  dcandisfld(handle, "rt", 3, dhx, dhy, dhz, 0, rt);
  dcandisfld(handle, "r", 3, dhx, dhy, dhz, 0, r);
  dcandisfld(handle, "rc", 3, dhx, dhy, dhz, 0, rc);
  dcandisfld(handle, "rr", 3, dhx, dhy, dhz, 0, rr);
  dcandisfld(handle, "vx", 3, dhx, dhy, dhz, 0, vx);
  dcandisfld(handle, "vy", 3, dhx, dhy, dhz, 0, vy);
  dcandisfld(handle, "vz", 3, dhx, dhy, dhz, 0, vz);
  dcandisfld(handle, "p", 3, dhx, dhy, dhz, 0, p);
  dcandisfld(handle, "kk", 3, dhx, dhy, dhz, 0, kk);
  dcandisfld(handle, "kv", 3, dhx, dhy, dhz, 0, kv);
  dcandisfld(handle, "entcuflux", 3, dhx, dhy, dhz, 0, entcuflux);
  dcandisfld(handle, "entcufluxcum", 3, dhx, dhy, dhz, 0, entcufluxcum);
  dcandisfld(handle, "rtcuflux", 3, dhx, dhy, dhz, 0, rtcuflux);
  dcandisfld(handle, "rtcufluxcum", 3, dhx, dhy, dhz, 0, rtcufluxcum);
  dcandisfld(handle, "rrcuflux", 3, dhx, dhy, dhz, 0, rrcuflux);
  dcandisfld(handle, "rrcufluxcum", 3, dhx, dhy, dhz, 0, rrcufluxcum);
  dcandisfld(handle, "vxcuflux", 3, dhx, dhy, dhz, 0, vxcuflux);
  dcandisfld(handle, "vxcufluxcum", 3, dhx, dhy, dhz, 0, vxcufluxcum);
  dcandisfld(handle, "vycuflux", 3, dhx, dhy, dhz, 0, vycuflux);
  dcandisfld(handle, "vycufluxcum", 3, dhx, dhy, dhz, 0, vycufluxcum);
  dcandisfld(handle, "sentrad", 3, dhx, dhy, dhz, 0, sentrad);
  dcandisfld(handle, "sentradcum", 3, dhx, dhy, dhz, 0, sentradcum);
  dcandisfld(handle, "srtrain", 3, dhx, dhy, dhz, 0, srtrain);
  dcandisfld(handle, "srtraincum", 3, dhx, dhy, dhz, 0, srtraincum);

  /* legacy variables */
  dcandisfld(handle, "the0", 1, dhz, 0, 0, 0, the0);
  dcandisfld(handle, "thes0", 1, dhz, 0, 0, 0, thes0);
  dcandisfld(handle, "the", 3, dhx, dhy, dhz, 0, the);
  dcandisfld(handle, "thes", 3, dhx, dhy, dhz, 0, thes);

  /* write out the candis file */
  candiswrite(handle);
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

void wtgcomp(double sobel)
{
  
  /* wtg heating term from which the wtg vertical velocity is derived */
  for (iz = 0; iz < nz; iz++) {
    thpwtg[iz] = thmean[iz] - th0[iz];
  }
  
  /* compute vertical gradient of theta using centered difference
   * method and keep gradient above minthgrad
   */
  for (iz = 1; iz < nz - 1; iz++) {
    thetagrad[iz] = (thmean[iz + 1] - thmean[iz - 1])/(2.*dz);
    if (thetagrad[iz] < minthgrad) thetagrad[iz] = minthgrad;
  }
  
  /* extrapolate thetagrad to endpoints for spectral wtg (second
     order extrapolation can be dangerous here) */
  thetagrad[0] = thetagrad[1];
  thetagrad[nz - 1] = thetagrad[nz - 2];
  
  /* compute buoyancy frequency (original wtg ==> N = constant) */
  if (spectral) {
    brunt = 0.;
    bcount = 0;
    for (iz = 0; iz < nz; iz++) {
      z = iz*dz;
      if (z <= tpause) {
	brunt += pow(fabs(GEE*thetagrad[iz]/thmean[iz]),0.5);
	bcount++;
      }
    }
    brunt = brunt/bcount;
  }
  else {
    brunt = 1.0e-2;
  }
  
  /* compute Fourier coefficients */
  for (j = 0; j < nj; j++) {
    thetaj[j] = 0.;
    for (iz = 0; iz < nj; iz++) {
      
      /* use spectral wtg */
      if (spectral) {
	thetaj[j] += sine0[SI(iz,j)]*thpwtg[iz]*2./((nj + 1)*thetagrad[iz]);
      }
      
      /* use original wtg */
      else {
	thetaj[j] += sine0[SI(iz,j)]*adjmask[iz]*thpwtg[iz]*2./
	  ((nj + 1)*thetagrad[iz]);
      }
    }
  }
  
  /* compute wtg vertical velocity as Fourier sum */
  for (iz = 0; iz < nz; iz++) {
    
    /* zero wtg vertical velocity at this level */
    vzwtg[iz] = vzwtg2[iz] = 0.;
    
    /* wtg vertical velocity is zero above tropopause */
    if (iz < nj) {
      
      /* sum thetaj-weighted sine functions at each level -- j = 0 is trivial */
      for (j = 1; j < nj; j++) {
	
	/* revert to original wtg if desired (1/j -> 1) */
	if (spectral) {
	  vzwtg[iz] += thetaj[j]*sine0[SI(iz,j)]/j;
	}
	else {
	  vzwtg[iz] += thetaj[j]*sine0[SI(iz,j)];
	}
      }

      /* compute the original vzwtg without masking, BL treatment */
      vzwtg2[iz] = thpwtg[iz]/thetagrad[iz];

      /* normalize both properly */
      vzwtg[iz] *= brunt*tpause/(PIE*hscale);
      vzwtg2[iz] *= brunt*tpause/(PIE*hscale);
    }
  }
  
  /* fix original wtg vertical velocity in boundary layer by linear
   * interpolation between the surface and the assumed boundary
   * layer top -- only used for non-spectral model */
  if (!spectral) {
    izwtg = wtgbase/dz + 0.5;
    if (izwtg < 1) izwtg = 1;
    if (izwtg > nz - 1) izwtg = nz - 1;
    for (iz = 0; iz < izwtg; iz++) {
      vzwtg[iz] = (iz*vzwtg[izwtg])/izwtg;
    }
  }

  /* compute divergence term, extrapolating to endpoints */
  for (iz = 1; iz < nz - 1; iz++) {
    dvzwtgdz[iz] = (rho0[iz + 1]*vzwtg[iz + 1] -
		    rho0[iz - 1]*vzwtg[iz - 1])/(2.*rho0[iz]*dz);
  }
  dvzwtgdz[0] = 2.*dvzwtgdz[1] - dvzwtgdz[2];
  dvzwtgdz[nz - 1] = 2.*dvzwtgdz[nz - 2] - dvzwtgdz[nz - 3];
  
  /* smooth the divergence */
  for (iz = 1; iz < nz - 1; iz++) {
    dtemp[iz] = 0.5*dvzwtgdz[iz] + 0.25*(dvzwtgdz[iz + 1] +
					 dvzwtgdz[iz - 1]);
  }
  for (iz = 1; iz < nz - 1; iz++) dvzwtgdz[iz] = dtemp[iz];
  
  /* vertical advection of moisture and entropy -- upstream diff */
  for (iz = 1; iz < nz - 1; iz++) {
    if (vzwtg[iz] > 0.) {
      srtwtg[iz] += vzwtg[iz]*(rtmean[iz] - rtmean[iz - 1])/dz;
      sentwtg[iz] += vzwtg[iz]*(entmean[iz] - entmean[iz - 1])/dz;
    }
    else {
      srtwtg[iz] += vzwtg[iz]*(rtmean[iz + 1] - rtmean[iz])/dz;
      sentwtg[iz] += vzwtg[iz]*(entmean[iz + 1] - entmean[iz])/dz;
    }
  }
  
  /* make the horizontal moisture and entropy source terms either
   * lateral entrainment or a relaxation term
   */
  if (sobel > 0.) {
    
    /* imposed relaxation a la Adam Sobel */
    for (iz = 0; iz < nz; iz++) {
      srtwtg[iz] += sobel*(rtmean[iz] - rt0[iz]);
      sentwtg[iz] += sobel*(entmean[iz] - ent0[iz]);
    }
  }
  if (entrain) {
    
    /* lateral entrainment a la Raymond et al. */
    for (iz = 0; iz < nz; iz++) {
      if (dvzwtgdz[iz] > 0.) {
	srtwtg[iz] += (rtmean[iz] - rt0[iz])*dvzwtgdz[iz];
	sentwtg[iz] += (entmean[iz] - ent0[iz])*dvzwtgdz[iz];
      }
    }
  }
}

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

void doradiation()
{

  /* radiation control logic -- if fixed radiation specified */
  if (rcalcint == 0) {
    
    /* radiation specified in profile file and fixed profile specified
     * -- don't do radiation but take it from profile file
     */
    if (doprofile) {
      dorad = 0;
      
      /* transfer specified radiation to the radiation source term */
      for (ix = 0; ix < nxs; ix++) {
	for (iy = 0; iy < nys; iy++) {
	  for (iz = 0; iz < nz; iz++) {
	    i = I(ix,iy,iz);
	    sentrad[i] = sentrad0[iz];
	  }
	}
      }
    }
    
    /* radiation not specified in profile file -- use simple fixed radiation */
    else {
      for (iz = 0; iz < nz; iz++) {
	sentrad0[iz] = 0.;
      }
      dorad = 1;
    }
  }
  
  /* interactive radiation turned on */
  else {

    /* zero radiation from profile file to avoid confusion */
    for (iz = 0; iz < nz; iz++) sentrad0[iz] = 0.;
    
    /* time to do interactive radiation */
    if (++radcount > rcalcint) {
      radcount = 0;
      dorad = 1;
    }
    
    /* skip interactive radiation on this time step */
    else {
      dorad = 0;
    }
  }
  
  /* do actual radiation calculations */
  if (dorad) {
      
    /* loop on I columns */
    for (ix = 0; ix < nxs; ix++) {
      for (iy = 0; iy < nys; iy++) {
	
	/* compute the Exner function in column */
	for (iz = 0; iz < nz; iz++) {
	  i = I(ix,iy,iz);
	  zckm[iz] = iz*dz/1000.;
	  pi[iz] = exner(p[i]);
	  satr[iz] = rs(th[i],pi[iz]);
	}
	i = I(ix,iy,0);
	
	/* interactive radiation */
	if (rcalcint) {
	  radiation(nz,0,cld,stemp[k],ALBEDO,zckm,&th[i],&ent[i],
		    &rt[i],pi,&rho[i],satr,&sentrad[i],radflux);

	}

	/* fixed radiation */
	else {
	  fradiation(nz,zckm,pi,&th[i],radcool,tpause/1000.,radbrk,&ent[i],
		     &sentrad[i],&sentrad[i],radflux,radflux);
	}

	/* convert radiation rate from per kilosecond to per second */
	for (iz = 0; iz < nz; iz++) {
	  i = I(ix,iy,iz);
	  sentrad[i] = sentrad[i]/1000.;
	}
      }
    }
  }
}




/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/* main -- entry point */
int main(int argc, char *argv[])
{

  /* enable core dumps on floating point exceptions
     (model won't compile on UCAR machines with this line) */
  if (SYS == 0) {
    feenableexcept(FE_DIVBYZERO | FE_INVALID | FE_OVERFLOW );
  }

  /* usage */
  if (argc < 2 || argc > 4) {
    fprintf(stderr,"Usage: conv start_file [new_nt] [new_mt]\n");
    exit(21);
  }

  /* initialize_main -- do a bunch of initialize mpi stuff */
  MPI_Init(&argc,&argv);
  initialize_main();

  /* get start file */
  if (argc == 4) {
    strcpy(startfilename,argv[argc - 3]);
  }
  else if (argc == 3) {
    strcpy(startfilename,argv[argc - 2]);
  }
  else if (argc == 2) {
    strcpy(startfilename,argv[argc - 1]);
  }
  else {
    fprintf(stderr,"conv: you shouldn't get here!\n");
    exit(22);
  }
  
  /* open start file and get file names */
  if ((startfile = fopen(startfilename,"r")) == (FILE*)NULL) {
    fprintf(stderr,"conv: start file %s does not exist\n", startfilename);
    exit(23);
  }
  getsvar("basename",basename);
  getsvar("checkname",checkfilename);
  getsvar("profname",profname);

  /* if profile or checkfile names equal "-", these files don't exist */
  doprofile = strcmp("-",profname);
  docheckstart = strcmp("-",checkfilename);

  /* construct output checkfile name */
  sprintf(outcheckname,"checkfile.%s",basename);

  /* set initial value of mtold */
  mtold = 0;

  /* start from a checkpoint file if master process */
  if (docheckstart) {

    /* get stuff from checkpoint file after rank-related delay */
    sleep(1*rank);
    readcheck();

    /* get profile data if needed */
    if (doprofile) readprofile(nz, profname);

    /* reset values of nt, mt if 2nd, 3rd arguments exist */
    if (argc > 2) {
      if (argc == 4) {
	nt = atoi(argv[argc - 2]);
	mtold = mt;
	mt = atoi(argv[argc - 1]);
	if (rank == 0) {
	  fprintf(stderr,"Using new value of nt = %d\n",nt);
	  fprintf(stderr,"Using new value of mt = %d\n",mt);
	}
      }
      
      /* only reset value of nt */
      else if (argc == 3) {
	nt = atoi(argv[argc - 1]);
	if (rank == 0) {
	  fprintf(stderr,"Using new value of nt = %d\n",nt);
	}
      }
    }
  }

  /* otherwise, normal start */
  else {

    /* rh through vy2 are only used if profname = "-" except for tpause --
     * that is used to compute the adjustment mask */
    rh = getfvar("rh");
    tpause = getfvar("tpause");
    stab = getfvar("stab");
    vx1 = getfvar("vx1");
    vx2 = getfvar("vx2");
    vx3 = getfvar("vx3");
    vy1 = getfvar("vy1");
    vy2 = getfvar("vy2");

    /* get the parallelization grid information -- each slave process gets
     * one rectangular region with nx/gx by ny/gy grid points -- thus
     * we need gx*gy slave processes -- gx and gy must be 1 or a positive
     * even integer */
    gx = getivar("gx");
    gy = getivar("gy");
    nx = getivar("nx");
    ny = getivar("ny");

    /* check consistency of grid size and compute nxs, nys */
    consistency();

    /* get rest of parameters */
    nz = getivar("nz");
    nt = getivar("nt");
    mt = getivar("mt");
    lt = getivar("lt");
    dx = getfvar("dx");
    dy = getfvar("dy");
    dz = getfvar("dz");
    dt = getfvar("dt");
    localth = getfvar("localth");
    localrh = getfvar("localrh");
    xs = getfvar("xs");
    ys = getfvar("ys");
    zs = getfvar("zs");
    ranseed = getivar("ranseed");
    randomth = getfvar("randomth");
    randomrh = getfvar("randomrh");
    rhswitch = getfvar("rhswitch");
    eddy = getfvar("eddy");
    filter = getfvar("filter");
    lambda = getfvar("lambda");
    drh = getfvar("drh");
    crain = getfvar("crain");
    csnow = getfvar("csnow");
    cevap = getfvar("cevap");
    wtermw = getfvar("wtermw");
    wtermi = getfvar("wtermi");
    cld = getfvar("cld");
    radcool = getfvar("radcool");
    radbrk = getfvar("radbrk");
    rcalcint = getivar("rcalcint");
    cdrag = getfvar("cdrag");
    drelax = getfvar("drelax");
    wtg = getivar("wtg");
    entrain = getivar("entrain");
    sobel = getfvar("sobel");
    wtgbase = getfvar("wtgbase");
    minthgrad = getfvar("minthgrad");
    spectral = getivar("spectral");
    hscale = getfvar("hscale");
    stemp0 = getfvar("stemp0");
    dstemp = getfvar("dstemp");
    ueffmin = getfvar("ueffmin");
    mumax = getfvar("mumax");
    fclose(startfile);

    /* seed the random number generator -- default value = 1 */
    srandom(ranseed);
    
    /* get profile data if needed */
    if (doprofile) readprofile(nz, profname);

    /* define greatest Fourier mode number for spectral wtg */
    nj = (int)(tpause/dz + 0.5);
    if (spectral < 0) spectral = 0;
    if (spectral > 1) spectral = 1;
    
    /* set field sizes */
    nsize = nx*ny*nz;
    ssize = nx*ny;
    meanbuffsize = 5*nz + 1;  /* reflects the number of fields averaged */
    
    /* allocate space */
    allocate_main();

    /* initialize the sine eigenfunctions */
    for (iz = 0; iz < nj; iz++) {
      for (j = 0; j < nj; j++) {
	sine0[SI(iz,j)] = sin((double)(iz*j)*PIE/(double)(nj + 1));
      }
    }

    /* initialize the adjustment mask and zero some stuff */
    for (iz = 0; iz < nz; iz++) {
      if (iz < nj) adjmask[iz] = sine0[SI(iz,1)];
      else adjmask[iz] = 0.;
      vzwtg[iz] = vzwtg2[iz] = dvzwtgdz[iz] = thetagrad[iz] = thpwtg[iz] = 0.;
    }
    brunt = 0.;

    /* initialize thetaj along j axis */
    for (j = 0; j < nj; j++) {
      thetaj[j] = 0.;
    }

    /* set up the reference profile */
    if (doprofile) {

      /* pre-existing profile for thermodynamic initializations if needed */
      getprofile((double)0., th0, rt0, vx0, vy0, sentrad0, &stemp0);
      hydrostatic2(nz, dz, PSURF, th0, ent0, ents0, rho0, rt0);
    }
    else {
      
      /* no pre-existing profile -- do hydrostatic profile initialization */
      hydrostatic(nz, dz, PSURF, stemp0, rh, tpause, stab, th0, ent0, ents0,
		  rho0, rt0);
      
      /* initialize the wind profile and zero sentrad0 */
      for (iz = 0; iz < nz; iz++) {
	z = iz*dz;
	vx0[iz] = vx1 + vx2*z + 0.5*vx3*z*z;
	vy0[iz] = vy1 + vy2*z;
	
	/* constant wind above tropopause */
	if (z > tpause && iz > 0) {
	  vx0[iz] = vx0[iz - 1];
	  vy0[iz] = vy0[iz - 1];
	}
	sentrad0[iz] = 0.;
      }
    }

    /* more initializations */
    for (iz = 0; iz < nz; iz++) {
      z = iz*dz;
      
      /* zero the sink fields */
      sentwtg[iz] = srtwtg[iz] = thpwtg[iz] = svxwtg[iz] = svywtg[iz] =
	sentwtgcum[iz] = srtwtgcum[iz] = thpwtgcum[iz] = svxwtgcum[iz] =
	svywtgcum[iz] = 0.;
      
      /* stratospheric damping */
      if (z < tpause) {
	mu[iz] = 0.;
      }
      else {
	mu[iz] = mumax*(z - tpause)/(dz*nz - tpause);
      }
    }

    /* initialize 2-D and 3-D fields */
    it = 0;
    time = 0.;
    for (ix = 0; ix < nx; ix++) {
      for (iy = 0; iy < ny; iy++) {
      
	mask = makemask(ix,iy);

	/* make random potential temperature and RH perturbations */
	thperturb = randomth*((double)(random())/((double)RAND_MAX) - 0.5);
	rhperturb = randomrh*((double)(random())/((double)RAND_MAX) - 0.5);
	for (iz = 0; iz < nz; iz++) {
	  x = (ix*dx - 0.5*((nx - 1)*dx))/xs;
	  y = (iy*dy - 0.5*((ny - 1)*dy))/ys;
	  z = (iz*dz)/zs;
	  zscale = z;

	  /* this computes the vertical structure of the perturbations */
	  thinitpert = (1. + localth*exp(-x*x - y*y - z*z))*
	    (1. + thperturb*zscale*exp(1. - zscale));
	  rhinitpert = (1. + localrh*exp(-x*x - y*y - z*z))*
	    (1. + rhperturb*zscale*exp(1. - zscale))*rhswitch;
	  i = G(ix,iy,iz);

	  /* if mask is one, set the result */
	  if (mask > 0.5) {
	    vx[i] = vx0[iz];
	    vy[i] = vy0[iz];
	    vz[i] = 0.;
	    th[i] = th0[iz]*thinitpert;
	    rho[i] = rho0[iz]/thinitpert;
	    rt[i] = rt0[iz]*rhinitpert;
	    p[i] = P_REF*pow(rho[i]*R*th[i]/P_REF,GAMMA);
	    ent[i] = entcomp(th[i],rt[i],exner(p[i]));
	    ents[i] = entscomp(th[i],exner(p[i]));
	  }

	  /* otherwise, set to zero */
	  else {
	    vx[i] = 0.;
	    vy[i] = 0.;
	    vz[i] = 0.;
	    th[i] = 0.;
	    rho[i] = 0.;
	    rt[i] = 0.;
	    p[i] = 0.;
	    ent[i] = 0.;
	    ents[i] = 0.;
	  }

	  /* this stuff is zeroed as it is not initialized with non-zero vals */
	  kk[i] = kv[i] = r[i] = rc[i] = rr[i] = entcuflux[i] =
	    entcufluxcum[i] = rtcuflux[i] = rtcufluxcum[i] = rrcuflux[i] =
	    rrcufluxcum[i] = vxcuflux[i] = vxcufluxcum[i] = vycuflux[i] =
	    vycufluxcum[i] = sentrad[i] = srtrain[i] = sentradcum[i] =
	    srtraincum[i] = 0.;
	}

	/* zero some 2-D stuff */
	k = H(ix,iy);
	entflux[k] = entfluxcum[k] = rtflux[k] = rtfluxcum[k] =
	  vxflux[k] = vxfluxcum[k] = vyflux[k] = vyfluxcum[k] = 
	  rainflux[k] = rainfluxcum[k] = 0.;
      }
    }

    /* initialize radiation count */
    radcount = rcalcint;

    /* compute the STEMP distribution */
    stempcalc(nx,ny,dstemp,stemp0,stemp);

    /* write initial file */
    if (rank == 0) {
      docandis(basename, profname, startfilename, it, nx, ny, nz, nj, dx, dy,
	       dz, dt, time, rho, th, ent, ents, rt, r, rc, rr, vx, vy, vz,
	       p, kk, kv, entcuflux, entcufluxcum, rtcuflux, rtcufluxcum,
	       rrcuflux, rrcufluxcum, vxcuflux, vxcufluxcum, vycuflux,
	       vycufluxcum, sentrad, sentradcum, srtrain, srtraincum,
	       th0, ent0, ents0, rho0, rt0, vx0, vy0, sentrad0,
	       sentwtg, srtwtg,thpwtg, sentwtgcum, srtwtgcum, thpwtgcum,
	       vzwtg, dvzwtgdz, adjmask, svxwtg, svxwtgcum, svywtg, svywtgcum,
	       mu, entflux, entfluxcum, rtflux,
	       rtfluxcum, vxflux, vxfluxcum, vyflux, vyfluxcum,
	       rainflux, rainfluxcum, stemp, brunt, thetaj, vzwtg2,
	       the, thes, the0, thes0);
    }

    /* starting values for main loop */
    itstart = 1;
  }
  
  /* set up x and y loop limits for computations */
  if (mpisize > 0) {
    nx1 = nxs*(igx - 1);
    ny1 = nys*(igy - 1);
  }
  else {
    nx1 = ny1 = 0;
  }

  /* set up stuff for superloop -- now there are three loop levels,
   * the major loop, the minor loop, and the super loop, which only
   * runs the radiation and timestep code -- thus, WTG, etc., are
   * not called for every time step
   */

  /* major loop */
  for (it = itstart; it <= nt; it++) {

    /* if mt has been reset, use different time counting scheme */
    if (mtold > 0) {
      
      /* set initial value of time based on checkstart file */
      if (it == itstart) time = it*mtold*lt*dt;
      time += mt*lt*dt;
    }
    else {
      time = it*mt*lt*dt;
    }

    /* minor loop */
    for (jt = 1 - mt; jt <= 0; jt++) {
      finetime = time + jt*lt*dt;

      /* modify the reference profile in time-dependent fashion and
       * fix hydrostatic balance at each step
       */

      if (doprofile) {
	getprofile(finetime, th0, rt0, vx0, vy0, sentrad0, &stemp0);
	hydrostatic2(nz, dz, PSURF, th0, ent0, ents0, rho0, rt0);
	stempcalc(nx,ny,dstemp,stemp0,stemp);
      }

      /* moisture and ent sink terms -- first zero */
      for (iz = 0; iz < nz; iz++) {
	sentwtg[iz] = srtwtg[iz] = 0.;
      }

      /* do WTG calculations if desired */
      if (wtg) {
	
	/* average over the whole domain and bring results to master */
	average_main();

	/* do the core of WTG */
	wtgcomp(sobel);

	/* wind terms */
	for (iz = 0; iz < nz; iz++) {
	  svxwtg[iz] = (vxmean[iz] - vx0[iz])*drelax;
	  svywtg[iz] = (vymean[iz] - vy0[iz])*drelax;
	  
	  /* compute the cumulative source terms */
	  sentwtgcum[iz] += sentwtg[iz]*dt*lt;
	  srtwtgcum[iz] += srtwtg[iz]*dt*lt;
	  thpwtgcum[iz] += thpwtg[iz]*dt*lt;
	  svxwtgcum[iz] += svxwtg[iz]*dt*lt;
	  svywtgcum[iz] += svywtg[iz]*dt*lt;
	}
      }

      /* I am a worker or a lone master process -- execute superloop */
      if (mpisize == 0 || rank > 0) {
	for (kt = 0; kt < lt; kt++) {
	  doradiation();
	  timestep(mpisize, gx, gy, mycom, rank, igx, igy, nx, nxs, nx1,
		   ny, nys, ny1, nz, dx, dy, dz, dt, eddy, filter, lambda,
		   drh, crain, csnow, cevap, wtermw, wtermi, cld, cdrag,
		   ueffmin, sentwtg, srtwtg, svxwtg, svywtg, mu, vx0,
		   vy0, psmean, rho, vx, vy, vz, p, th, ent, rt, r, rc, rr,
		   kk, kv, vxcuflux, vxcufluxcum, vycuflux, vycufluxcum,
		   entcuflux, entcufluxcum, rtcuflux, rtcufluxcum,
		   rrcuflux, rrcufluxcum, sentrad, sentradcum, srtrain,
		   srtraincum, entflux, entfluxcum, rtflux, rtfluxcum,
		   vxflux, vxfluxcum, vyflux, vyfluxcum, rainflux,
		   rainfluxcum, stemp);
	}
      }
    }

    /* gather all the output if multiprocessing */
    gather_main();

    /* do output if master process */
    if (rank == 0) {
      
      /* compute the saturated equivalent potential temperature */
      for (ix = 0; ix < nx; ix++) {
	for (iy = 0; iy < ny; iy++) {
	  for (iz = 0; iz < nz; iz++) {
	    i = G(ix,iy,iz);
	    ents[i] = entscomp(th[i],exner(p[i]));
	  }
	}
      }
      
      /* write an output file */
      docandis(basename, profname, startfilename, it, nx, ny, nz, nj, dx, dy,
	       dz, dt, time, rho, th, ent, ents, rt, r, rc, rr, vx, vy, vz,
	       p, kk, kv, entcuflux, entcufluxcum, rtcuflux, rtcufluxcum,
	       rrcuflux, rrcufluxcum, vxcuflux, vxcufluxcum, vycuflux,
	       vycufluxcum, sentrad, sentradcum, srtrain, srtraincum,
	       th0, ent0, ents0, rho0, rt0, vx0, vy0, sentrad0,
	       sentwtg, srtwtg, thpwtg, sentwtgcum, srtwtgcum, thpwtgcum,
	       vzwtg, dvzwtgdz, adjmask, svxwtg, svxwtgcum, svywtg, svywtgcum,
	       mu, entflux, entfluxcum, rtflux,
	       rtfluxcum, vxflux, vxfluxcum, vyflux, vyfluxcum,
	       rainflux, rainfluxcum, stemp, brunt, thetaj, vzwtg2,
	       the, thes, the0, thes0);

      /* write a checkpoint file */
      docheck();
    }
  }

  MPI_Finalize();
  exit(0);
}
