#!/bin/sh
#
# crmcufluxes.sh -- Compute the vertical profiles of entropy, total
# cloud water, and rain water fluxes due to convection.  Profile
# files are the input and the cumulative results over a specified
# time interval are used to compute the fluxes.  Also put in the
# critical forcing information.
# This is for entropy028 and later.
#
if test $# != 9
then
    echo "Usage: crmcufluxes.sh case_number vwind sst thm0 thm2 thm4 rhm0 rhm2 rhm4"
else
    time1=2000000
    time2=5000000
    ztrop=15000
    bldepth=1000
    num=$1
    vwind=$2
    sst=$3
    thm0=$4
    thm2=$5
    thm4=$6
    rhm0=$7
    rhm2=$8
    rhm4=$9

    dtime=$(($time2 - $time1))
    infile=prof.$num
    outfile=fluxes.$num
    plotfileprefix=pltfluxes_${num}
    plotfile=${plotfileprefix}.pdf

    cdfrdim time $time1 < $infile | \
	cdfextr sentradcum sentwtgcum srtraincum srtwtgcum rainfluxcum \
	entfluxcum rtfluxcum entcufluxcum rtcufluxcum rrcufluxcum rho p th \
	vzwtg th0 | cdfextr -s z > time1.$$
    cdfrdim time $time2 < $infile | \
	cdfextr sentradcum sentwtgcum srtraincum srtwtgcum rainfluxcum \
	entfluxcum rtfluxcum entcufluxcum rtcufluxcum rrcufluxcum rho | \
	cdfextr -s z > time2.$$

    cdfmerge time2.$$ '' time1.$$ '' | \
	cdfmath "sentradcum sentradcum. - $dtime / sentrad =" | \
	cdfmath "sentwtgcum sentwtgcum. - $dtime / sentwtg =" | \
	cdfmath "srtraincum srtraincum. - $dtime / srtrain =" | \
	cdfmath "srtwtgcum srtwtgcum. - $dtime / srtwtg =" | \
	cdfmath "entfluxcum entfluxcum. - $dtime / entsurfflux =" | \
	cdfmath "rtfluxcum rtfluxcum. - $dtime / rtsurfflux =" | \
	cdfmath "rainfluxcum rainfluxcum. - $dtime / rainsurfflux =" | \
	cdfmath "entcufluxcum entcufluxcum. - $dtime / entcuflux =" | \
	cdfmath "rtcufluxcum rtcufluxcum. - $dtime / rtcuflux =" | \
	cdfmath "rrcufluxcum rrcufluxcum. - $dtime / rrcuflux =" | \
	cdfextr sentrad sentwtg srtrain srtwtg rainsurfflux \
	entsurfflux rtsurfflux rho entcuflux rtcuflux rrcuflux \
	p th vzwtg th0 | \
	cdfextr -s z | \
	cdfmath "rho sentrad * rhosentrad =" | \
	cdfmath "rho sentwtg * rhosentwtg =" | \
	cdfmath "rho srtwtg * rhosrtwtg =" | \
	cdfmath "rho srtrain * rhosrtrain =" | \
	cdfinteg -t entradflux rhosentrad z | \
	cdfinteg -t entwtgflux rhosentwtg z | \
	cdfinteg -t srtrainflux rhosrtrain z | \
	cdfinteg -t rtwtgflux rhosrtwtg z | \
	cdfmath "entradflux -1 * entradflux =" | \
	cdfmath "entwtgflux -1 * entwtgflux =" | \
	cdfmath "srtrainflux -1 * srtrainflux =" | \
	cdfmath "rtwtgflux -1 * rtwtgflux =" | \
	cdfmath "entwtgflux entradflux - entcuflux2 =" | \
	cdfmath "rtwtgflux srtrainflux - rtcuflux2 =" | \
	cdfmath "srtrainflux raincuflux2 =" | \
	cdfmath "$num num =" | \
	cdfmath "$vwind vwind =" | \
	cdfmath "$sst sst =" | \
	cdfmath "$thm0 thm0 =" | \
	cdfmath "$thm2 thm2 =" | \
	cdfmath "$thm4 thm4 =" | \
	cdfmath "$rhm0 rhm0 =" | \
	cdfmath "$rhm2 rhm2 =" | \
	cdfmath "$rhm4 rhm4 =" | \
	crmfluxfit3.sh entcuflux $ztrop $bldepth | \
	crmfluxfit3.sh entradflux $ztrop $bldepth | \
	crmfluxfit3.sh entwtgflux $ztrop $bldepth | \
	crmfluxfit3.sh rtcuflux $ztrop $bldepth | \
	crmfluxfit3.sh rrcuflux $ztrop $bldepth | \
	crmfluxfit3.sh rtwtgflux $ztrop $bldepth > $outfile

    rm time1.$$ time2.$$

    qplot "12,fs/6,8,2,1,fg/1,sp/entcuflux2,z,b,2,convection,p/entradflux,z,r,2,radiation,p/entwtgflux,z,g,2,wtg,p/entcuflux,z,ko,2,convraw,p/entcuflux_fit,z,b--,2,,p/entwtgflux_fit,z,g--,2,,p/0:0,0:20000,k,1,d/entropy flux (J~/K~/m^2~/s),lx/elevation (m),ly/1,gr/ne,le/Case $num time average [$time1-$time2] s,tl/2,sp/rtcuflux2,z,b,2,rt,p/raincuflux2,z,r,2,rain,p/rtwtgflux,z,g,2,wtg,p/rtcuflux,z,ko,2,rtraw,p/rrcuflux,z,ko,2,rainraw,p/rtcuflux_fit,z,b--,2,,p/rtwtgflux_fit,z,g--,2,,p/rrcuflux_fit,z,r--,2,,p/0:0,0:20000,k,1,d/rt and rain flux (kg~/m^2~/day),lx/elevation (m),ly/nw,le/1,gr" < $outfile $plotfileprefix pdf

#xpdf ${plotfile}
fi
