#!/bin/sh
#
# crmfluxaug.sh -- Compute surface flux augmentation factors in CRM
# model relative the SCM fluxes.  Both entropy and total cloud water
# factors are computed.  The starting point is a surf file.  Assumes
# a 2-D CRM run with specified u and v wind components.
#
if test $# != 6
then
    echo "Usage: crmfluxaug.sh case_num time1 time2 uwind vwind stemp"
else
    num=$1
    time1=$2
    time2=$3
    us=$4
    vs=$5
    stemp=$6
    infile=surf.$num
    W=3.
    CP=1005.
    TREF=300.
    L=2.833e6
    
    cdfmath "$us $us * $vs $vs * $W $W * + + 0.5 pow uscm =" < $infile | \
	cdfmath "p 0 * $stemp + tsurf =" | \
	cdfmath "p 100 / pmb =" | \
	cdfmath "tsurf 273.15 - tsurfc =" | \
	cdfmr pmb tsurfc tsurfc rsatsurf | \
	cdfmath "rsatsurf 1000 / rsatsurf =" | \
	cdfmath "100000 p / 0.286 pow tsurf * thsurf =" | \
	cdfmath "$L $CP $TREF * / alfa =" | \
	cdfmath "thsurf $TREF / log alfa rsatsurf * + $CP * entsatsurf =" | \
	cdfmath "entsatsurf ent - dentcrm =" | \
	cdfmath "rsatsurf rt - drtcrm =" | \
	cdfmath "vx vx * vy vy * $W $W * + + 0.5 pow ucrm =" | \
	cdfmath "ucrm dentcrm * nument =" | \
	cdfmath "ucrm drtcrm * numrt =" | \
	cdfmath "uscm dentcrm * denoment =" | \
	cdfmath "uscm drtcrm * denomrt =" | \
	cdfrdim x 0 500000 | \
	cdfrdim time $time1 $time2 | \
	cdfmath "nument denoment / aent =" | \
	cdfmath "numrt denomrt / art =" | \
	cdflook
fi
