#!/bin/sh
#
# crmfluxfit3.sh -- Do a 3 parameter power law fit to a selected
# flux profile.
#
if test $# != 3
then
    echo "crmfluxfit3.sh field ztrop bldepth"
else

# get the command line stuff
    field=$1
    ztrop=$2
    bldepth=$3

# name the fit parameters and output field
    a0=${field}_0
    a1=${field}_1
    a2=${field}_2
    outfield=${field}_fit

# do the calculation, windowing on troposphere -- input from stdin
    cdfwindow z 0 $ztrop > temp$$
    cdfmerge temp$$ '' | \
	cdfmath "1 z $ztrop / - zoff =" | \
	cdfmath "zoff zoff * W =" | \
	cdfmath "z $bldepth / -1 * exp A =" | \
	cdfmath "zoff zoff * B =" | \
	cdfmath "$field W * Y0 =" | \
	cdfmath "$field W * A * Y1 =" | \
	cdfmath "$field W * B * Y2 =" | \
	cdfmath "W W * X00 =" | \
	cdfmath "X00 A * X01 =" | \
	cdfmath "X00 B * X02 =" | \
	cdfmath "X01 X10 =" | \
	cdfmath "X10 A * X11 =" | \
	cdfmath "X01 B * X12 =" | \
	cdfmath "X02 X20 =" | \
	cdfmath "X12 X21 =" | \
	cdfmath "X20 B * X22 =" | \
	cdfmean z m Y0 Y1 Y2 X00 X01 X02 X10 X11 X12 X20 X21 X22 | \
	cdfmath "X00m X11m X22m * * X10m X21m X02m * * + D =" | \
	cdfmath "D X01m X12m X20m * * + X20m X11m X02m * * - D =" | \
	cdfmath "D X10m X01m X22m * * - X21m X12m X00m * * - D =" | \
	cdfmath "Y0m X11m X22m * X21m X12m * - * D0 =" | \
	cdfmath "D0 Y1m X21m X02m * X01m X22m * - * + D0 =" | \
	cdfmath "D0 Y2m X01m X12m * X11m X02m * - * + D0 =" | \
	cdfmath "Y0m X12m X20m * X22m X10m * - * D1 =" | \
	cdfmath "D1 Y1m X22m X00m * X02m X20m * - * + D1 =" | \
	cdfmath "D1 Y2m X02m X10m * X12m X00m * - * + D1 =" | \
	cdfmath "Y0m X10m X21m * X20m X11m * - * D2 =" | \
	cdfmath "D2 Y1m X20m X01m * X00m X21m * - * + D2 =" | \
	cdfmath "D2 Y2m X00m X11m * X10m X01m * - * + D2 =" | \
	cdfmath "D0 D / $a0 =" | \
	cdfmath "D1 D / $a1 =" | \
	cdfmath "D2 D / $a2 =" | \
	cdfmath "$a1 A * Z1 =" | \
	cdfmath "$a2 B * Z2 =" | \
	cdfmath "W $a0 Z1 Z2 + + * $outfield =" | \
	cdfextr -r W A B Y0 Y1 Y2 X00 X01 X02 X10 X11 X12 X20 X21 X22 \
	Y0m Y1m Y2m X00m X01m X02m X10m X11m X12m X20m X21m X22m \
	D D0 D1 D2
    rm temp$$
fi
