#!/bin/sh
#
# crmmakeref.sh -- Make a reference profile with specified characteristics.
# This takes a previous 1-D file as a starting point on the standard input.
# The wave functions are of the form evident below.  They have been
# renormalized for orthonormality on half the domain by multiplying by
# 2^1/2.  The constants are of the form
#         Cn = 1/[(pi/4)^1/2 2^n n!]^1/2
# where n = 0, 2, 4.  These forms have been verified to be orthonormal
# with zmult = 1.  (For zmult != 1, the length squared is zmult.)
# Reference: Any quantum mechanics text, or see Wikipedia on the harmonic
# oscillator.  The actual th and rt profiles are linear superpositions of
# these modes.  This is a filter, with rce mean file on the standard input
# and the new init file on the standard output.  The th, rt, vx, vy,
# stherad, and stemp parameters are included.
#
if test $# != 9
then
    echo "Usage: crmmakeref.sh uwind vwind thm0 thm2 thm4 rhm0 rhm2 rhm4 stemp"
else
    uwind=$1
    vwind=$2
    thm0=$3
    thm2=$4
    thm4=$5
    rhm0=$6
    rhm2=$7
    rhm4=$8
    stemp=$9

    zmult=4000
    C0=1.0623
    C2=0.3756
    C4=0.05421

# create init file
    cdfmath "z $zmult / zs =" | \
	cdfmath 'zs zs * -0.5 * exp efn =' | \
	cdfmath "efn $C0 * mode0 =" | \
	cdfmath "efn 4 zs * zs * 2 - * $C2 * mode2 =" | \
	cdfmath "efn 16 zs * zs * zs * zs * 48 zs * zs * - 12 + * $C4 * mode4 =" | \
	cdfmath "mode0 $thm0 * mode2 $thm2 * mode4 $thm4 * + + thm =" | \
	cdfmath "mode0 $rhm0 * mode2 $rhm2 * mode4 $rhm4 * + + rhm =" | \
	cdfextr th thm rt rhm sentrad stemp | \
	cdfextr -s x y z | \
	cdfmath "z 0 * $vwind + vy =" | cdfmath "z 0 * $uwind + vx =" | \
	cdfmath "th thm + th =" | \
	cdfmath "1 rhm + rt * rt =" | \
	cdfmath "$stemp stemp =" | \
	cdfcomment "th coeffs: $thm0 $thm2 $thm4" | \
	cdfcomment "rh coeffs: $rhm0 $rhm2 $rhm4"
fi
