#!/bin/sh
#
# crmrunit.sh -- Run a conv job under MPI in a separate working
# directory note that nprocs should equal gx*gy + 1 where gx and gy
# are found in the in.??? file.  The host file should also have nprocs
# cores available.  (Fussy, fussy!)
# This version is set up to use the MPICH MPI package.  (On Arch, the
# openMPI is the default.)  Note that the host file format is
# hostname:8 for 8 slots in MPICH, unlike the hostname slots=8 format
# of openMPI.
#
if test $# != 5
then
    echo "Usage: crmrunit.sh host nprocs run_number model_version restart(y/n)"
else

# stuff needing editing
    WORKINGDIR=/data.gy/raymond/convtest
    BINDIR=/home.gryphon/raymond/bin.amd64

# get the command line arguments
    hostnum=$1
    procs=$2
    num=$3
    version=$4
    restart=$5

# construct the hostfile name, the input and initialization file names
    hostname=mpichhost-$hostnum
    infile=in.$num
    initfile=init.$num

# an init file must exist even if it is empty
    touch $initfile

# set up for a separate working directory -- edit as needed
    workingdir=${WORKINGDIR}/t$num
    if test "$restart" = 'y'
    then
	sed -e "s/checkname\t-\t/checkname\tcheckfile.test$num\t/" < $infile > junk$$
	mv $infile ${infile}.bak
	mv junk$$ $infile
    else
	rm -rf $workingdir
	mkdir $workingdir
    fi
    cp $hostname $infile $initfile $workingdir
    cd $workingdir

# start the run
    /opt/mpich/bin/mpiexec -f $hostname -n $procs $BINDIR/$version $infile &
fi
