#!/bin/sh
#
# crmngms.sh -- Compute the normalized gross moist stability for
#   CRM run in WTG mode.  Starts from CRM profile files, as
#   calculated by crmprof.sh.  The cumulative files are used
#   because they are far more accurate in expressing true time
#   averages, since they gather source terms at every time step,
#   not just the time steps used for creating the output files.
# Version 1 does the full calculation as the vertical integral of
#   sentwtg divided by the vertical integral of srtwtg.
# Version 2 of the ngms assumes a steady state and is calculated
#   as the entropy forcing from surface fluxes and radiation
#   divided by the net rainfall rate, as calculated from srtrain.
# Both versions are averaged over the specified time interval.
#
if test $# != 3
then
    echo "Usage: crmngms.sh time1 time2 run_num"
else
    time1=$1
    time2=$2
    num=$3

    infile=prof.$num
    L=2.833e6
    TREF=300.

    cdfderiv entflux2 entfluxcum time < $infile | \
	cdfderiv rtflux2 rtfluxcum time | \
	cdfderiv rhosentrad rhosentradcum time | \
	cdfderiv rhosrtrain rhosrtraincum time | \
	cdfderiv rhosentwtg rhosentwtgcum time | \
	cdfderiv rhosrtwtg rhosrtwtgcum time | \
	cdfdefint z | \
	cdfmath "rhosentwtg $TREF * num1 =" | \
	cdfmath "rhosrtwtg $L * -1 * denom1 =" | \
	cdfmath "rhosentrad entflux2 + $TREF * num2 =" | \
	cdfmath "rhosrtrain rtflux2 + $L * -1 * denom2 =" | \
	cdfrdim time $time1 $time2 | \
	cdfmath "num1 denom1 / ngms1 =" | \
	cdfmath "num2 denom2 / ngms2 =" | \
	cdflook | egrep 'ngms1 = |num1 = |denom1 = |ngms2 = |num2 = |denom2 = '
fi
