#!/bin/sh
#
# crmstats.sh -- Compute rainfall rate, satfrac, and instability index.
#
if test $# != 3
then
    echo "Usage: crmstats time1 time2 run_num"
else
    time1=$1
    time2=$2
    infile=prof.$3
    TREF=300
    CP=1005

# rainfall rate and saturation fraction
    cdfmath "th $TREF / log $CP * entd =" < $infile | \
	cdfmath "ent entd - rho * sfnum =" | \
	cdfmath "ents entd - rho * sfdenom =" | \
	cdfdefint z | \
	cdfrdim time $time1 $time2 | \
	cdfmath "sfnum sfdenom / satfrac =" | \
	cdfmath "rainflux -86400 * rainrate =" | \
	cdflook | \
	egrep 'rainrate = |satfrac = '

# saturation fraction in reference profile
    cdfmath "th0 $TREF / log $CP * entd0 =" < $infile | \
	cdfmath "ent0 entd0 - rho * sfnum0 =" | \
	cdfmath "ents0 entd0 - rho0 * sfdenom0 =" | \
	cdfdefint z | \
	cdfrdim time $time1 $time2 | \
	cdfmath "sfnum0 sfdenom0 / satfrac0 =" | \
	cdflook | \
	egrep 'satfrac0 = '

# instability index
    cdfrdim z 1000 3000 < $infile | \
	cdfextr ents | cdfextr -s time | \
	cdfrdim time $time1 $time2 > lowband

    cdfrdim z 5000 7000 < $infile | \
	cdfextr ents | cdfextr -s time | \
	cdfrdim time $time1 $time2 > highband
    
    cdfmerge lowband '' highband '' | \
	cdfmath "ents ents. - instabindex =" | \
	cdflook | egrep "instabindex = "

# instability index in reference profile
    cdfrdim z 1000 3000 < $infile | \
	cdfextr ents0 | cdfextr -s time | \
	cdfrdim time $time1 $time2 > lowband

    cdfrdim z 5000 7000 < $infile | \
	cdfextr ents0 | cdfextr -s time | \
	cdfrdim time $time1 $time2 > highband
    
    cdfmerge lowband '' highband '' | \
	cdfmath "ents0 ents0. - instabindex0 =" | \
	cdflook | egrep "instabindex0 = "

    rm lowband highband
fi
