#!/bin/sh
#
# crmxstats.sh -- Compute rainfall rate, satfrac, instability index,
# DCIN, and NGMS.  Also compute two radiation parameters.
#
if test $# != 9
then
    echo "Usage: crmxstats run_num vwind sst thm0 thm2 thm4 rhm0 rhm2 rhm4"
else
    runnum=$1
    vwind=$2
    sst=$3
    thm0=$4
    thm2=$5
    thm4=$6
    rhm0=$7
    rhm2=$8
    rhm4=$9
    
    infile=prof.$runnum
    outfile=stats.$runnum
    time1=2000000
    time2=5000000
    dtime=$(($time2 - $time1))
    TREF=300
    CP=1005
    L=2.833e6
    CD=0.001
    EPSILON=0.622
    PREF=100000
    W=3

    
    # time-mean surface fluxes
    cdfrdim time $time1 < $infile | \
	cdfextr sentradcum sentwtgcum srtraincum srtwtgcum rainfluxcum \
	entfluxcum rtfluxcum entcufluxcum rtcufluxcum rrcufluxcum p th \
	rt ent ents vzwtg th0 ent0 ents0 rt0 | cdfextr -s time z > time1.$$
    cdfrdim time $time2 < $infile | \
	cdfextr sentradcum sentwtgcum srtraincum srtwtgcum rainfluxcum \
	entfluxcum rtfluxcum entcufluxcum rtcufluxcum rrcufluxcum rho | \
	cdfextr -s time z > time2.$$
    cdfmerge time2.$$ '' time1.$$ '' | \
	cdfmath "entfluxcum entfluxcum. - $dtime / entsurfflux =" | \
	cdfmath "rtfluxcum rtfluxcum. - $dtime / 86400 * rtsurfflux =" | \
	cdfmath "rainfluxcum rainfluxcum. - $dtime / -86400 * rainsurfflux =" | \
	cdfextr rainsurfflux entsurfflux rtsurfflux | \
	cdfextr -s time > temp0
    rm time1.$$ time2.$$

    # rainfall rate and saturation fraction -- also incorporate command line
    cdfmath "th $TREF / log $CP * entd =" < $infile | \
	cdfmath "ent entd - rho * sfnum =" | \
	cdfmath "ents entd - rho * sfdenom =" | \
	cdfdefint z | \
	cdfrdim time $time1 $time2 | \
	cdfmath "sfnum sfdenom / satfrac =" | \
	cdfextr satfrac > temp1

    # saturation fraction in reference profile and integrated radiation
    cdfmath "th0 $TREF / log $CP * entd0 =" < $infile | \
	cdfmath "ent0 entd0 - rho * sfnum0 =" | \
	cdfmath "ents0 entd0 - rho0 * sfdenom0 =" | \
	cdfmath "rho sentrad * sentrad_all =" | \
	cdfdefint z | \
	cdfrdim time $time1 $time2 | \
	cdfmath "sfnum0 sfdenom0 / satfrac0 =" | \
	cdfextr satfrac0 sentrad_all > temp2

    # instability index
    cdfrdim z 1000 3000 < $infile | \
	cdfextr ents | cdfextr -s time | \
	cdfrdim time $time1 $time2 > lowband

    cdfrdim z 5000 7000 < $infile | \
	cdfextr ents | cdfextr -s time | \
	cdfrdim time $time1 $time2 > highband
    
    cdfmerge lowband '' highband '' | \
	cdfmath "ents ents. - instabindex =" | \
	cdfextr instabindex > temp3

    # instability index in reference profile
    cdfrdim z 1000 3000 < $infile | \
	cdfextr ents0 | cdfextr -s time | \
	cdfrdim time $time1 $time2 > lowband

    cdfrdim z 5000 7000 < $infile | \
	cdfextr ents0 | cdfextr -s time | \
	cdfrdim time $time1 $time2 > highband
    
    cdfmerge lowband '' highband '' | \
	cdfmath "ents0 ents0. - instabindex0 =" | \
	cdfextr instabindex0 > temp4

    # DCIN
    cdfrdim z 0 1000 < $infile | \
	cdfextr ent | cdfextr -s time | \
	cdfrdim time $time1 $time2 > lowband

    cdfrdim z 1500 1800 < $infile | \
	cdfextr ents | cdfextr -s time | \
	cdfrdim time $time1 $time2 > highband
    
    cdfmerge lowband '' highband '' | \
	cdfmath "ents ent - dcin =" | \
	cdfextr dcin > temp8

    # DCIN in reference profile
    cdfrdim z 0 1000 < $infile | \
	cdfextr ent0 | cdfextr -s time | \
	cdfrdim time $time1 $time2 > lowband

    cdfrdim z 1500 1800 < $infile | \
	cdfextr ents0 | cdfextr -s time | \
	cdfrdim time $time1 $time2 > highband
    
    cdfmerge lowband '' highband '' | \
	cdfmath "ents0 ent0 - dcin0 =" | \
	cdfextr dcin0 > temp9

    # DCIN2
    cdfrdim z 0 1500 < $infile | \
	cdfextr ent | cdfextr -s time | \
	cdfrdim time $time1 $time2 > lowband

    cdfrdim z 2000 4500 < $infile | \
	cdfextr ents | cdfextr -s time | \
	cdfrdim time $time1 $time2 > highband
    
    cdfmerge lowband '' highband '' | \
	cdfmath "ents ent - dcin2 =" | \
	cdfextr dcin2 > temp11

    # DCIN2 in reference profile
    cdfrdim z 0 1500 < $infile | \
	cdfextr ent0 | cdfextr -s time | \
	cdfrdim time $time1 $time2 > lowband

    cdfrdim z 2000 4500 < $infile | \
	cdfextr ents0 | cdfextr -s time | \
	cdfrdim time $time1 $time2 > highband
    
    cdfmerge lowband '' highband '' | \
	cdfmath "ents0 ent0 - dcin20 =" | \
	cdfextr dcin20 > temp12

    # NGMS and command line parameters
    cdfderiv entflux2 entfluxcum time < $infile | \
	cdfderiv rtflux2 rtfluxcum time | \
	cdfderiv rhosentrad rhosentradcum time | \
	cdfderiv rhosrtrain rhosrtraincum time | \
	cdfderiv rhosentwtg rhosentwtgcum time | \
	cdfderiv rhosrtwtg rhosrtwtgcum time | \
	cdfdefint z | \
	cdfmath "rhosentwtg $TREF * num1 =" | \
	cdfmath "rhosrtwtg $L * -1 * denom1 =" | \
	cdfmath "rhosentrad entflux2 + $TREF * num2 =" | \
	cdfmath "rhosrtrain rtflux2 + $L * -1 * denom2 =" | \
	cdfrdim time $time1 $time2 | \
	cdfmath "num1 denom1 / ngms1 =" | \
	cdfmath "num2 denom2 / ngms2 =" | \
	cdfextr num1 denom1 ngms1 num2 denom2 ngms2 | \
	cdfmath "$thm0 thm0 =" | \
	cdfmath "$thm2 thm2 =" | \
	cdfmath "$thm4 thm4 =" | \
	cdfmath "$rhm0 rhm0 =" | \
	cdfmath "$rhm2 rhm2 =" | \
	cdfmath "$rhm4 rhm4 =" | \
	cdfmath "$runnum runnum =" | \
	cdfmath "$vwind vwind =" | \
	cdfmath "$sst sst =" > temp5

    # mass flux at two levels
    cdfrdim z 3000 5000 < $infile | \
	cdfmath "rho vzwtg * mfluxlo =" | \
	cdfextr mfluxlo | cdfextr -s time | \
	cdfrdim time $time1 $time2 > lowband

    cdfrdim z 5000 7000 < $infile | \
	cdfmath "rho vzwtg * mfluxhi =" | \
	cdfextr mfluxhi | cdfextr -s time | \
	cdfrdim time $time1 $time2 > highband

    cdfmerge lowband '' highband '' | \
	cdfmath "mfluxlo mfluxhi - 0.5 * mfluxdiff =" | \
	cdfmath "mfluxlo mfluxhi + 0.5 * mfluxmean =" > temp6

    # low-level radiation
    cdfwindow z 0 4000 < $infile | \
	cdfmath "rho sentrad * sentrad_4km =" | \
	cdfdefint z | \
	cdfrdim time $time1 $time2 | \
	cdfextr sentrad_4km > temp7

    # reference surface entropy flux
    cdfrdim z 0 < $infile | cdfrdim time 0 | \
	cdfmath "$L $CP $TREF * / alfa =" | \
	cdfmath "17.67 stemp 273.15 - * stemp 29.65 - / exp 611.2 * es =" | \
	cdfmath "$EPSILON es * $PREF / rsss =" | \
	cdfmath "stemp $TREF / log alfa rsss * + $CP * entsss =" | \
	cdfmath "vy0 vy0 * $W $W * + 0.5 pow ueff =" | \
	cdfmath "rho0 $CD * ueff * entsss ent0 - * entsurfflux0 =" | \
	cdfextr -s | cdfextr alfa es rsss entsss ueff entsurfflux0 > temp10

    # merge the stuff
    cdfmerge temp0 '' temp1 '' temp2 '' temp3 '' temp4 '' \
	     temp5 '' temp6 '' temp7 '' temp8 '' temp9 '' temp10 '' \
	     temp11 '' temp12 '' | \
	cdfextr -s > $outfile

    rm lowband highband temp0 temp1 temp2 temp3 temp4 temp5 temp6 temp7 \
       temp8 temp9
fi
