/* thermo.h -- Thermodynamic parameters
 */

/* thermodynamic and related constants */
#define GAMMA 1.4
#define KAPPA (1. - 1./GAMMA)
#define R 287.
#define P_REF 100000.
#define GEE 9.8
#define FREEZING 273.15
#define TREF 300.
#define RHOREF 1.2
#define CP 1005.0
#define LVAPOR 2.50e6
#define LFREEZ 3.33e5
#define ALFA ((LVAPOR + LFREEZ)/(CP*TREF))
#define EPSILON 0.622

/* the thermodynamic routines */
double rs(double th,double pi);  /* saturation mixing ratio (g/g) */
double escalc(double td);  /* saturation vapor pressure (pa) */
double thetacomp(double ent,double rt,double pi);  /* potential temp (K) */
double thetacomp2(double ent,double rt,double pi);  /* fast pot temp (K) */
double entcomp(double th,double rt,double pi);  /* entropy (J/kg/K) */
double entcomp2(double th,double rt,double rsat);  /* entropy (J/kg/K) */
double entscomp(double th,double pi);  /* saturated entropy (J/kg/K) */
double thecomp(double ent);   /* equivalent potential temperature */
double entrate(double therate,double the);  /* compute entropy rate */
double gmoist(double th,double pi);  /* moist adiabatic d th/dz (K/m) */
double exner(double p);  /* Exner function */
